﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    [Api(Description = "报告文件")]
    [Alias(Configs.TableNameReportFile)]
    public class ReportFile: EntityWithMetadata, IEntityInAssemblyDb
    {
        /// <summary>
        /// 报告号
        /// </summary>
        [Required]
        [ApiMember(Description = "所属报告Kvid")]
        public Guid ReportKvid { get; set; }

        /// <summary>
        /// 使用到的模板的Kvid,避免更新Report中的Template后，重新生成ReportFile
        /// </summary>
        [Required]
        [ApiMember(Description = "所属模板Kvid")]
        public Guid TemplateKvid { get; set; }

        /// <summary>
        /// 文件Kvid
        /// </summary>
        //[DefaultEmptyGuid]
        //[ApiMember(Description = "存储Kvid")]
        //public Guid DbFileKvid { get; set; }

        /// <summary>
        /// 模板Kvid
        /// </summary>
        [Required]
        [ApiMember(Description = "模板Kvid")]
        public Guid TemplateFileKvid { get; set; }

        /// <summary>
        /// 文件名称
        /// </summary>
        [StringLength(200),Required]
        [ApiMember(Description = "文件名称")]
        public string Name { get; set; }

        /// <summary>
        /// 文件标签,
        /// 设计规则：
        /// 在某一状态需要重新生成，则在状态值前加G
        /// 如：G100,代表Status为100时，重新生成文件；
        /// 200，代表Status为200时根据已经生成的文件基础上执行生成程序
        /// </summary>
        [Default("")]
        [StringLength(200)]
        [ApiMember(Description = "文件标签")]
        public string Tags { get; set; }

        /// <summary>
        /// 模板路径
        /// </summary>
        [StringLength(500),Required]
        [ApiMember(Description = "模板路径")]
        public string TemplateFilePath { get; set; }

        /// <summary>
        /// 排序号，一定要正确，合成报告时用到
        /// </summary>
        [Default(0)]
        [ApiMember(Description = "排序号")]
        public int SortID { get; set; }

        /// <summary>
        /// 生成的报告路径
        /// </summary>
        [StringLength(500)]
        [Default("")]
        [ApiMember(Description = "生成路径")]
        public string Path { get; set; }

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }
}
