﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    public enum TemplateDetailType
    {
        Unsupported = 0,
        /// <summary>
        /// 检测项目录,有子项的大项
        /// </summary>
        TestFolder,

        /// <summary>
        /// 检测项，子项或者无子项的大项
        /// </summary>
        Test
    }

    [Alias(Configs.TableNameTemplateDetail)]
    [Api(Description = "项目模版明细")]
    public class TemplateDetail : EntityWithMetadata, IEntityInAssemblyDb
    {
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        #region 检验项目信息
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }

        [ApiMember(Description = "关联父级Kvid")]
        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }

        [ApiMember(Description = "引用标准Kvid")]
        [DefaultEmptyGuid]
        public Guid StandardKvid { get; set; }

        [ApiMember(Description = "标准代码")]
        [Default("")]
        [StringLength(200)]
        public string StandardCode { get; set; }

        [ApiMember(Description = "标准名称")]
        [Default("")]
        [StringLength(200)]
        public string StandardTitle { get; set; }

        [ApiMember(Description = "引用检测项目Kvid")]
        [DefaultEmptyGuid]
        public Guid DetectionKvid { get; set; }

        [ApiMember(Description = "引用检测项目名称")]
        [Default("")]
        [StringLength(200)]
        public string DetectionTitle { get; set; }

        [ApiMember(Description = "测试方法")]
        [Default("")]
        [StringLength(200)]
        public string DetectionCodeActual { get; set; }

        [Default("")]
        [ApiMember(Description = "数值单位")]
        [StringLength(100)]
        public string DetectionUnit { get; set; }

        [ApiMember(Description = "标准值")]
        [StringLength(500)]
        public string DetectionValueQualified { get; set; }

        [ApiMember(Description = "引用原生检测项目Kvid")]
        [DefaultEmptyGuid]
        public Guid DetectionReferenceKvid { get; set; }
        #endregion

        #region 金额相关
        [ApiMember(Description = "货币单位")]
        [IgnoreUpdate]
        [Default("")]
        [StringLength(20)]
        public CurrencyUnit Currency { get; set; }

        [ApiMember(Description = "条目Kvid")]
        [DefaultEmptyGuid]
        public Guid GoodsKvid { get; set; }

        [ApiMember(Description = "包装Kvid")]
        [DefaultEmptyGuid]
        public Guid PackageKvid { get; set; }

        [ApiMember(Description = "价格类型")]
        [Default("")]
        public string PackageUnit { get; set; }

        [ApiMember(Description = "价格")]
        [DecimalLength(10, 2), Default(0)]
        public decimal Amount { get; set; }
        #endregion

        #region 基本信息
        [ApiMember(Description = "检验项目类型")]
        public TemplateDetailType Type { get; set; }

        [ApiMember(Description = "项目名称", IsRequired = true)]
        [Required]
        [StringLength(500)]
        public string Title { get; set; }

        [ApiMember(Description = "首要排序号")]
        [Default(0)]
        public int SortId { get; set; }

        [ApiMember(Description = "次要排序号")]
        [Default(0)]
        public int SortIdEx { get; set; }

        [ApiMember(Description = "备注")]
        [StringLength(2000)]
        public string Remark { get; set; }
        #endregion

        #region 检测人员信息
        /// <summary>
        /// 录入人员与检测人员的区别：
        /// 录入人员指实际录数据的人员
        /// 检测人员指实验还未开始，由相关部门将该检验分配给的指定检测人员
        /// </summary>

        [ApiMember(Description = "检测小组Kvid")]
        [DefaultEmptyGuid]
        public Guid WorkGroupKvid { get; set; }

        [ApiMember(Description = "检测小组")]
        [StringLength(50)]
        public string WorkGroupName { get; set; }

        [ApiMember(Description = "检测人员Kvid")]
        [DefaultEmptyGuid]
        public Guid WorkerKvid { get; set; }

        [ApiMember(Description = "检测人员")]
        [StringLength(50), Default("")]
        public string WorkerName { get; set; }
        #endregion

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }
}
