﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    /// <summary>
    /// 记录模板和文件的关系及标签
    /// </summary>
    [Api(Description = "模版文件")]
    [Alias(Configs.TableNameTemplateFile)]
    public class TemplateFile:EntityWithMetadata, IEntityInAssemblyDb
    {
        [Required]
        public Guid TemplateKvid { get; set; }

        [StringLength(200), Required]
        [ApiMember(Description = "文件名称")]
        public string Name { get; set; }

        /// <summary>
        /// 记录文件的物理路径
        /// </summary>
        [StringLength(200), Required]
        [IgnoreUpdate]
        [ApiMember(Description ="文件路径")]
        public string Path { get; set; }

        /// <summary>
        /// 用来记录当前模板在指定状态生成
        /// 设计规则：
        /// 1、记录Report的Status，字母+数字，用逗号分隔，如U100,G200,300
        /// 2、G+Status  重新生成
        /// 3、U+Status  在已有的文件上生成
        /// 若无字母前缀，默认在已有文件上生成
        /// </summary>
        [StringLength(200), Default("")]
        [ApiMember(Description = "文件标签")]
        public string Tags { get; set; }

        [Default(0)]
        [ApiMember(Description = "排序号")]
        public int SortId { get; set; }

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }
}
