﻿using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Transforms;
using Kivii.Linq;
using Kivii.Office.OpenXml.Excel.Templates;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

namespace Kivii.Lac.Lims.Extensions
{
    public static class ReportExtension
    {
        /// <summary>
        /// 更改报告部分信息,同步更新样品
        /// </summary>
        /// <param name="report"></param>
        /// <param name="originalReportId"></param>
        /// <param name="conn"></param>
        public static void UpdateSampleInfos(this Report report, string originalReportId, IDbConnection conn = null)
        {
            #region 数据录入
            report.ThrowIfNull("未传入更新报告!");
            (report.SampleKvid == Guid.Empty).ThrowIfTrue("未传入主样品信息!");
            #endregion
            bool disposeConn = conn == null;
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Sample>();
            var samples = conn.Select<Sample>(o=>o.RootKvid==report.SampleKvid);
            //获取主样品
            var sample = samples.FirstOrDefault(o=>o.RootKvid==o.Kvid);
            sample.ThrowIfNull("未找到对应主样品信息!");
            //获取子样品信息
            var sampleChildren = samples.Where(o=>o.ParentKvid==report.SampleKvid).ToList();
            if (report.Metadata.ContainsKey("SampleMode"))
            {
                sample.SourceType = report.Metadata["SampleMode"];
                sample.AddOnlyProperties(o => o.SourceType);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.SourceType = sample.SourceType;
                    sampleChild.AddOnlyProperties(o => o.SourceType);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleSourceBatchNumber))
            {
                //存在修改送样方式后样品编号置空的情况
                if (report.SampleSourceBatchNumber.IsEmpty())
                {
                    sample.SourceBatchNumber = null;
                }
                else
                {
                    sample.SourceBatchNumber = report.SampleSourceBatchNumber;
                }
                sample.AddOnlyProperties(o => o.SourceBatchNumber);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.SourceBatchNumber = sample.SourceBatchNumber;
                    sampleChild.AddOnlyProperties(o => o.SourceBatchNumber);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleName))
            {
                sample.Name = report.SampleName;
                sample.AddOnlyProperties(o => o.Name);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.Name = sample.Name;
                    sampleChild.AddOnlyProperties(o => o.Name);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleBatchNumber))
            {
                sample.BatchNumber = report.SampleBatchNumber;
                sample.AddOnlyProperties(o => o.BatchNumber);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.BatchNumber = sample.BatchNumber;
                    sampleChild.AddOnlyProperties(o => o.BatchNumber);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleBrand))
            {
                sample.Brand = report.SampleBrand;
                sample.AddOnlyProperties(o => o.Brand);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.Brand = sample.Brand;
                    sampleChild.AddOnlyProperties(o => o.Brand);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleModel))
            {
                sample.Model = report.SampleModel;
                sample.AddOnlyProperties(o => o.Model);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.Model = sample.Model;
                    sampleChild.AddOnlyProperties(o => o.Model);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleQuantity))
            {
                sample.Quantity = report.SampleQuantity;
                sample.AddOnlyProperties(o => o.Quantity);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.Quantity = sample.Quantity;
                    sampleChild.AddOnlyProperties(o => o.Quantity);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleQuantityUnit))
            {
                sample.QuantityUnit = report.SampleQuantityUnit;
                sample.AddOnlyProperties(o => o.QuantityUnit);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.QuantityUnit = sample.QuantityUnit;
                    sampleChild.AddOnlyProperties(o => o.QuantityUnit);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleGrade))
            {
                sample.Grade = report.SampleGrade;
                sample.AddOnlyProperties(o => o.Grade);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.Grade = sample.Grade;
                    sampleChild.AddOnlyProperties(o => o.Grade);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleGradeSafety))
            {
                sample.GradeSafety = report.SampleGradeSafety;
                sample.AddOnlyProperties(o => o.GradeSafety);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.GradeSafety = sample.GradeSafety;
                    sampleChild.AddOnlyProperties(o => o.GradeSafety);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleManufacturerKvid))
            {
                sample.ManufacturerKvid = report.SampleManufacturerKvid;
                sample.ManufacturerName = report.SampleManufacturerName;
                sample.AddOnlyProperties(o => new { o.ManufacturerKvid, o.ManufacturerName });
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.ManufacturerKvid = sample.ManufacturerKvid;
                    sampleChild.ManufacturerName = sample.ManufacturerName;
                    sampleChild.AddOnlyProperties(o => new { o.ManufacturerKvid, o.ManufacturerName });
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleManufacturerContactKvid))
            {
                sample.ManufacturerContactKvid = report.SampleManufacturerContactKvid;
                sample.ManufacturerContactName = report.SampleManufacturerContactName;
                sample.AddOnlyProperties(o => new { o.ManufacturerContactKvid, o.ManufacturerContactName });
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.ManufacturerContactKvid = sample.ManufacturerContactKvid;
                    sampleChild.ManufacturerContactName = sample.ManufacturerContactName;
                    sampleChild.AddOnlyProperties(o => new { o.ManufacturerContactKvid, o.ManufacturerContactName });
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleManufacturerContactNumber))
            {
                sample.ManufacturerContactNumber = report.SampleManufacturerContactNumber;
                sample.AddOnlyProperties(o => o.ManufacturerContactNumber);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.ManufacturerContactNumber = sample.ManufacturerContactNumber;
                    sampleChild.AddOnlyProperties(o => o.ManufacturerContactNumber);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.SampleManufacturerAddress))
            {
                sample.ManufacturerAddress = report.SampleManufacturerAddress;
                sample.AddOnlyProperties(o => o.ManufacturerAddress);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.ManufacturerAddress = sample.ManufacturerAddress;
                    sampleChild.AddOnlyProperties(o => o.ManufacturerAddress);
                }
            }
            if (report.OnlyPropertiesIsExist(o => o.ReportId))
            {
                sample.SerialNumber = report.ReportId;
                sample.AddOnlyProperties(o => o.SerialNumber);
                foreach (var sampleChild in sampleChildren)
                {
                    sampleChild.SerialNumber = originalReportId.IsNullOrEmpty()? report.ReportId: sampleChild.SerialNumber.Replace(originalReportId, report.ReportId);
                    sampleChild.AddOnlyProperties(o => o.SerialNumber);
                }
            }
            if (sample.OnlyProperties.Count > 0) conn.UpdateOnly(sample);
            foreach (var sampleChild in sampleChildren)
            {
                if (sampleChild.OnlyProperties.Count > 0) conn.UpdateOnly(sampleChild);
            }
            if (disposeConn) conn.Dispose();
        }

        /// <summary>
        /// 同步计算报告中的检验标准及项目相关字段
        /// 注意: 此方法只会进行统计赋值到TestStandards TestDetections字段中  不涉及到更新数据库数据
        /// </summary>
        /// <param name="report"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Report CalculateReportStandard(this Report report,IDbConnection conn=null)
        {
            //获取报告
            (report.Kvid == Guid.Empty).ThrowIfTrue("缺少ReportKvid无法同步检验标准及项目!");
            report.ThrowIfNull("未找到需要同步的报告!");
            bool disposeConn = conn == null;
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取ReportItem
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx);

            //整个报告的全部项目
            var allReportItems = conn.Select(queryReportItems);
            if (disposeConn) conn.Dispose();
            //全项中的大项
            var primaryAllReportItems = allReportItems.filterReportItem();
            if (primaryAllReportItems.IsNullOrEmpty())
            {
                report.TestStandards = "";
                report.AddOnlyProperties(o=>o.TestStandards);
                report.TestDetections = "";
                report.AddOnlyProperties(o=>o.TestDetections);
                return report;
            }
            //primaryAllReportItems.ThrowIfNullOrEmpty("此报告全部检验项目未找到检验大项,结构异常,请联系管理员!");

            //查看是否已经存在色号
            bool existSelf = allReportItems.Exists(o => o.Type == ReportItemType.SampleSelf);

            var testStandards = string.Empty;//检验依据
            var testDetections = string.Empty;//检验项目
            var testStandardsList = new List<string>();
            var testDetectionsList = new List<string>();

            #region 组合检验依据
            var groupStandards = primaryAllReportItems.GroupBy(o => new { o.StandardCode, o.StandardTitle });

            foreach (var kv in groupStandards)
            {
                testStandardsList.Add(kv.Key.StandardCode + (kv.Key.StandardTitle.IsNullOrEmpty() ? "" : "《" + kv.Key.StandardTitle + "》"));
            }
            testStandardsList = testStandardsList.Distinct().ToList();
            testStandards = String.Join("、", testStandardsList);
            report.TestStandards = testStandards;
            report.AddOnlyProperties(o => o.TestStandards);
            #endregion

            #region 组合检验项目
            //根据大项进行分组
            var groupPrimarys = primaryAllReportItems.GroupBy(o => o.Title);
            foreach (var kv in groupPrimarys)
            {
                var detectionStr = kv.Key;
                
                var self = new List<string>();
                var part = new List<string>();
                var actual = new List<string>();
                var method = new List<string>();

                //存在色号的情况
                if (existSelf)
                {
                    //key色号,value部位集合
                    var selfAndParts = new Dictionary<string, List<string>>();
                    foreach (var primary in kv)
                    {
                        var parentItem = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid);
                        //存在色号情况下 不可能为空  为空即跳过不处理
                        if (parentItem == null) continue;

                        var selfParent = parentItem;
                        if (parentItem.Type == ReportItemType.SamplePart) selfParent = allReportItems.FirstOrDefault(o => o.Kvid == parentItem.ParentKvid);

                        if (selfParent == null) continue;

                        var detectionMethodDescription = "";
                        if (primary.Metadata.ContainsKey("DetectionMethodDescription"))
                        {
                            detectionMethodDescription = primary.Metadata["DetectionMethodDescription"].IsNullOrEmpty() ? "" :primary.Metadata["DetectionMethodDescription"];
                        }
                        var detectionCodeActual = primary.DetectionCodeActual;
                        if (primary.Metadata.ContainsKey("DetectionCodeActual")) detectionCodeActual = primary.Metadata["DetectionCodeActual"];
                        if (!detectionMethodDescription.IsNullOrEmpty()) {
                            detectionCodeActual = detectionCodeActual.Replace(detectionMethodDescription, "");
                            detectionMethodDescription = " " + detectionMethodDescription;
                        }
                        if (!actual.Contains(primary.DetectionCodeActual)) actual.Add(detectionCodeActual);

                        if (!method.Contains(detectionMethodDescription)) method.Add(detectionMethodDescription);

                        //说明有部位
                        if (parentItem.Type == ReportItemType.SamplePart)
                        {
                            if (selfAndParts.ContainsKey(selfParent.Title))
                            {
                                if (!selfAndParts[selfParent.Title].Contains(parentItem.Title)) selfAndParts[selfParent.Title].Add(parentItem.Title);
                            }
                            else
                            {
                                var values = new List<string>();
                                values.Add(parentItem.Title);
                                selfAndParts.Add(selfParent.Title, values);
                            }
                        }
                        //没有部位
                        else
                        {
                            if (!selfAndParts.ContainsKey(selfParent.Title))
                            {
                                var values = new List<string>();
                                values.Add(string.Empty);
                                selfAndParts.Add(selfParent.Title, values);
                            }
                        }
                    }

                    foreach (var selfAndPart in selfAndParts)
                    {
                        self.Add(selfAndPart.Key);
                        selfAndPart.Value.RemoveAll(o => o.IsNullOrEmpty());
                        part.AddRange(selfAndPart.Value);
                    }
                }
                //没有色号的情况
                else
                {
                    foreach(var primary in kv)
                    {
                        
                        var detectionMethodDescription = "";
                        if (primary.Metadata.ContainsKey("DetectionMethodDescription"))
                        {
                            detectionMethodDescription = primary.Metadata["DetectionMethodDescription"].IsNullOrEmpty() ? "" :primary.Metadata["DetectionMethodDescription"];
                        }
                        var detectionCodeActual = primary.DetectionCodeActual;
                        if (primary.Metadata.ContainsKey("DetectionCodeActual")) detectionCodeActual = primary.Metadata["DetectionCodeActual"];
                        if (!detectionMethodDescription.IsNullOrEmpty())
                        {
                            detectionCodeActual = detectionCodeActual.Replace(detectionMethodDescription, "");
                            detectionMethodDescription = " " + detectionMethodDescription;
                        }
                        if (!actual.Contains(primary.DetectionCodeActual)) actual.Add(detectionCodeActual);
                        if (!method.Contains(detectionMethodDescription)) method.Add(detectionMethodDescription);

                        var parentItem = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid);
                        //说明有部位
                        if (parentItem != null)
                        {
                            if (!part.Contains(parentItem.Title))
                            {
                                part.Add(parentItem.Title);
                            }
                        }
                    }
                }

                self = self.Distinct().ToList();
                part = part.Distinct().ToList();
                actual = actual.Distinct().ToList();
                method = method.Distinct().ToList();

                if (!self.IsNullOrEmpty() && !part.IsNullOrEmpty())
                {
                    
                }

                detectionStr += (self.IsNullOrEmpty() ? string.Empty : $"[{string.Join("、", self)}]") + (part.IsNullOrEmpty() ? string.Empty : $"({string.Join("、", part)})") + (actual.IsNullOrEmpty() ? (method.IsNullOrEmpty() ? string.Empty : "(" + string.Join("、", method) + ")") : "(" + string.Join("、", actual) + string.Join("、", method) + ")");
                testDetectionsList.Add(detectionStr);
            }
            testDetections = String.Join("、", testDetectionsList);
            report.TestDetections = testDetections;
            report.AddOnlyProperties(o=>o.TestDetections);
            #endregion
            return report;
        }

        /// <summary>
        /// 同步计算报告中的检验费用相关字段合计
        /// 注意: 此方法只会进行统计赋值到AmountTest AmountTestPlan字段中  不涉及到更新数据库数据
        /// </summary>
        /// <param name="report"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static Report CalculateReportAmount(this Report report, IDbConnection conn = null)
        {
            //获取报告
            (report.Kvid == Guid.Empty).ThrowIfTrue("缺少ReportKvid无法同步检验项目金额!");
            report.ThrowIfNull("未找到需要同步的报告!");
            bool disposeConn = conn == null;
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取ReportItem
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortIdEx);
            var reportItems = conn.Select(queryReportItems);
            if (disposeConn) conn.Dispose();
            decimal amountTestPlan = 0;//理论收费
            decimal amountTest = 0;//实际收费
            foreach (var reportItem in reportItems)
            {
                if (reportItem.Type != ReportItemType.TestFolder && reportItem.Type != ReportItemType.Test) continue;
                //大项(未有上级目录)
                if (reportItem.ParentKvid == Guid.Empty)
                {
                    amountTestPlan += reportItem.AmountPlan;
                    amountTest += reportItem.Amount;
                    continue;
                }
                //大项(存在上级目录)
                var reportItemParent = reportItems.FirstOrDefault(o => o.Kvid == reportItem.ParentKvid);
                reportItemParent.ThrowIfNull("未找到对应检测项父节点,无法统计金额!");
                if (reportItem.Type == ReportItemType.TestFolder)
                {
                    (reportItemParent.Type != ReportItemType.Folder && reportItemParent.Type != ReportItemType.SamplePart && reportItemParent.Type != ReportItemType.SampleSelf).ThrowIfTrue("错误的目录结构!");
                    amountTestPlan += reportItem.AmountPlan;
                    amountTest += reportItem.Amount;
                }
                else
                {
                    if (reportItemParent.Type == ReportItemType.TestFolder) continue;
                    (reportItemParent.Type != ReportItemType.Folder && reportItemParent.Type != ReportItemType.SamplePart && reportItemParent.Type != ReportItemType.SampleSelf).ThrowIfTrue("错误的目录结构!");
                    amountTestPlan += reportItem.AmountPlan;
                    amountTest += reportItem.Amount;
                }
            }
            report.AmountTestPlan = amountTestPlan;// * report.UrgentRate;
            report.AddOnlyProperties(o=>o.AmountTestPlan);
            report.AmountTest = amountTest;// * report.UrgentRate;
            report.AddOnlyProperties(o=>o.AmountTest);
            report.QuotationAmount = report.QuotationAmount;// * report.UrgentRate;
            report.AddOnlyProperties(o=>o.QuotationAmount);
            return report;
        }
        
        /// <summary>
        /// 添加任务流转记录
        /// </summary>
        /// <param name="report"></param>
        /// <param name="Title">流转标题</param>
        /// <param name="From">当前流程</param>
        /// <param name="To">目标流程</param>
        /// <param name="Summary">摘要</param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static ReportNote AddReportNote(this Report report, string Title, string From, string To, string Summary, IDbConnection conn = null,string OperatorName = null)
        {
            report.ThrowIfNull("请传入要流转的报告!");
            report.Kvid.ThrowIfEmpty("请传入要流转的报告Kvid!");
            Title.ThrowIfNullOrEmpty("流转标题");
            From.ThrowIfNullOrEmpty("当前流程");
            To.ThrowIfNullOrEmpty("目标流程");
            bool disposeConn = conn == null;

            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<ReportNote>();

            var reportNote = new ReportNote();
            reportNote.ReportKvid = report.Kvid;
            reportNote.Title = Title;
            reportNote.Type = "Status";
            reportNote.From = From;
            reportNote.To = To;
            reportNote.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            reportNote.OperatorName = OperatorName.IsNullOrEmpty()?KiviiContext.CurrentMember.FullName: OperatorName;
            reportNote.OperateTime = DateTime.Now;
            reportNote.Summary = Summary;
            //reportNote.IP = KiviiContext.Request.UserHostAddress;
            conn.Insert(reportNote);
            reportNote.RemoveAllOnlyProperties();
            if (disposeConn) conn.Dispose();
            return reportNote;
        }

        /// <summary>
        /// 添加项目流转记录
        /// </summary>
        /// <param name="report"></param>
        /// <param name="Title">流转标题</param>
        /// <param name="From">当前流程</param>
        /// <param name="To">目标流程</param>
        /// <param name="Summary">摘要</param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static ReportNote AddReportItemNote(this Report report, string Title, string From, string To, string Summary, IDbConnection conn = null)
        {
            report.ThrowIfNull("请传入要流转的报告!");
            report.Kvid.ThrowIfEmpty("请传入要流转的报告Kvid!");
            Title.ThrowIfNullOrEmpty("流转标题");
            From.ThrowIfNullOrEmpty("当前流程");
            To.ThrowIfNullOrEmpty("目标流程");
            bool disposeConn = conn == null;

            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<ReportNote>();

            var reportNote = new ReportNote();
            reportNote.ReportKvid = report.Kvid;
            reportNote.Title = Title;
            reportNote.Type = "ReportItemStatus";
            reportNote.From = From;
            reportNote.To = To;
            reportNote.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            reportNote.OperatorName = KiviiContext.CurrentMember.FullName;
            reportNote.OperateTime = DateTime.Now;
            reportNote.Summary = Summary;
            //reportNote.IP = KiviiContext.Request.UserHostAddress;
            conn.Insert(reportNote);
            reportNote.RemoveAllOnlyProperties();
            if (disposeConn) conn.Dispose();
            return reportNote;
        }
        /// <summary>
        /// 添加项目流转记录
        /// </summary>
        /// <param name="report"></param>
        /// <param name="Title">流转标题</param>
        /// <param name="From">当前流程</param>
        /// <param name="To">目标流程</param>
        /// <param name="Summary">摘要</param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static ReportNote AddDataEntryBackNote(this Report report, string Title, string From, string To, string Summary, IDbConnection conn = null)
        {
            report.ThrowIfNull("请传入要流转的报告!");
            report.Kvid.ThrowIfEmpty("请传入要流转的报告Kvid!");
            Title.ThrowIfNullOrEmpty("流转标题");
            From.ThrowIfNullOrEmpty("当前流程");
            To.ThrowIfNullOrEmpty("目标流程");
            bool disposeConn = conn == null;

            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<ReportNote>();

            var reportNote = new ReportNote();
            reportNote.ReportKvid = report.Kvid;
            reportNote.Title = Title;
            reportNote.Type = "DataEntryBack";
            reportNote.From = From;
            reportNote.To = To;
            reportNote.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            reportNote.OperatorName = KiviiContext.CurrentMember.FullName;
            reportNote.OperateTime = DateTime.Now;
            reportNote.Summary = Summary;
            //reportNote.IP = KiviiContext.Request.UserHostAddress;
            conn.Insert(reportNote);
            reportNote.RemoveAllOnlyProperties();
            if (disposeConn) conn.Dispose();
            return reportNote;
        }
        /// <summary>
        /// 报告流程进度描述
        /// </summary>
        /// <param name="report"></param>
        /// <returns></returns>
        public static string AssignDescription(int Status)
        {
            string description = "";
            switch (Status)
            {
                case (int)ReportStatusType.CommissionAccept:
                    description = "委托受理";
                    break;
                case 999:
                    description = "管理部审核";
                    break;
                case (int)ReportStatusType.TaskAssign:
                    description = "任务分派";
                    break;
                case (int)ReportStatusType.DataEntry:
                    description = "项目检验";
                    break;
                case (int)ReportStatusType.ReportPreparation:
                    description = "报告编制";
                    break;
                case (int)ReportStatusType.ReportReview:
                    description = "报告审核";
                    break;
                case (int)ReportStatusType.ReportIssue:
                    description = "报告签发";
                    break;
                case (int)ReportStatusType.ReportPrint:
                    description = "报告打印";
                    break;
                case int.MaxValue:
                    description = "报告归档";
                    break;
                default:
                    description = "报告流程异常，请与管理员联系";
                    break;
            }
            return description;
        }

        #region 委托下达相关方法
        /// <summary>
        /// 委托协议下达处理  100->委托受理 至 300->任务分派 至 400->项目检验
        /// 此方法针对100-300-400使用,项目检验下达之后流程不处理
        /// 参考 apps.jst-gov.com\Jst.Biz.Lims\Extensions\ContractExtension.cs 方法ContractFlowGo
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
        public static Report Assign(this Report report, ReportStatusType statusType, IDbConnection conn = null, bool isCompleteReport = true)
        {
            report.ThrowIfNull("请传入要下达的报告信息!");
            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            if (!isCompleteReport)
            {
                report = conn.SingleById<Report>(report.Kvid);
                report.ThrowIfNull("未找到对应的报告信息!");
            }
            //如果当前的报告已经不再项目检验了，不能对这份单子做任何的处理
            if (report.Status >= (int)ReportStatusType.DataEntry) return report;

            //var queryReportItems = conn.From<ReportItem>().Where(o => o.ReportKvid == report.Kvid);
            //var reportItems = conn.Select(queryReportItems);
            var reportItems = report.GetOriginalReportItems(conn);
            if (reportItems.IsNullOrEmpty()) throw new Exception($"当前委托协议({report.ReportId})还未选择检验项目，请待选择检验项目后重新操作！");
            #region 验证子检测项目数据结构完整性
            var reportItemKvids = reportItems.Select(o => o.Kvid).ToList();
            //2021/10/17新增取消验证部位项目，脏数据
            (reportItems.Exists(o => o.ParentKvid != Guid.Empty && !Sql.In(o.ParentKvid, reportItemKvids) && o.Type != ReportItemType.SamplePart)).ThrowIfTrue("当前检验项目中有子项检验不对应大项检验的情况，如不是在委托受理阶段请联系系统管理员！");
            #endregion

            reportItems.RemoveAll(o => Sql.In(o.Type, ReportItemType.SampleSelf, ReportItemType.SamplePart, ReportItemType.Unsupported));
            if (reportItems.IsNullOrEmpty()) throw new Exception($"当前委托协议({report.ReportId})还未选择检验项目，请待选择检验项目后重新操作！");

            //****************************************************
            //此处初始的status的状态 还需判断是否是打回项目进行任务下达,否则打回到委托受理状态的报告无法通过此方法下达
            int status = (int)ReportStatusType.TaskAssign;

            //获取未分组检验项目的个数
            var unGrouped = reportItems.filterReportItem().Where(o => o.WorkGroupKvid == Guid.Empty || o.WorkGroupName.IsNullOrEmpty()).Count();
            //获取未完成检验项目个数(只要没有判定结果的视为未检测)
            var unTested = reportItems.Where(o => o.Type == ReportItemType.Test & o.TestValue.IsNullOrEmpty()).Count();

            (report.Status == (int)ReportStatusType.TaskAssign && unGrouped > 0).ThrowIfTrue($"当前委托协议({report.ReportId})还未分组，请待检验项目全部分组后重新操作！");

            if (report.Status == (int)ReportStatusType.CommissionAccept)
            {
                //加强验证，无模板的kvid，无法下达
                report.TemplateKvid.ThrowIfEmpty("报告模板选择异常，请重新选择报告模板！如重复遇到，请联系系统管理员！");
                //如果有未分组的项目，就走到分派
                if (unGrouped > 0|| report.TaskAssignDate == null)
                {
                    status = (int)ReportStatusType.TaskAssign;
                }
                //如果是数据录入打回报告，直接到打回的地方
                //else if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom"))
                //{
                //    status = (int)ReportStatusType.DataEntry;
                //}
                //如果所有任务都完成直接到编制
                else if (!reportItems.filterReportItem().Exists(o => o.Status != int.MaxValue))
                {
                    status = (int)ReportStatusType.ReportPreparation;
                    //如果没有分派日期，同时更新分派日期
                    if (report.TaskAssignDate == null)
                    {
                        report.TaskAssignDate = DateTime.Now;
                        report.AddOnlyProperties(o => o.TaskAssignDate);
                    }
                }
                else
                {
                    status = (int)ReportStatusType.DataEntry;
                }
            }

            if (report.BizType == "X")
            {

                status = 700;
            }
            if (report.BizType == "G0")
            {

                status = 700;
            }

            //定义全局变量updateWorkGroupReportItems,用于分派统一检验组
            List<ReportItem> updateWorkGroupReportItems = null;
            if (report.Status == (int)ReportStatusType.TaskAssign)
            {
                //如果所有任务都完成直接到编制
                if (!reportItems.filterReportItem().Exists(o => o.Status != int.MaxValue))
                {
                    status = (int)ReportStatusType.ReportPreparation;
                }
                else
                {
                    status = (int)ReportStatusType.DataEntry;
                    //2021-10-17新增数据处理，处理子项的检验组合大项不一样的情况。
                    updateWorkGroupReportItems = reportItems.SyncWorkGroup();
                }
                report.TaskAssignDate = DateTime.Now;
                report.AddOnlyProperties(o => o.TaskAssignDate);

            }
            report.Status = status;
            report.AddOnlyProperties(o => o.Status);
            report.TestStartDate = (report.TestStartDate != null && report.TestStartDate > (DateTime?)DateTime.MinValue) ? report.TestStartDate : DateTime.Now;
            report.AddOnlyProperties(o => o.TestStartDate);

            //无论是打回的报告还是新建下达的报告，只要是经过受理下达、分派下达，都清空打回标记
            //if (report.Metadata.ContainsKey("FlowCallBack"))
            //{
            //    report.Metadata.Remove("FlowCallBack");
            //    report.AddOnlyProperties(o => o.Metadata);
            //}
            #region 任务下达时，判断是否有打回信息，按需清除打回信息
            if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
            {
                report.Metadata["CallBackTo"] = null;
                report.AddOnlyProperties(o => o.Metadata);
            }
            if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
            {
                report.Metadata["CallBackFrom"] = null;
                report.AddOnlyProperties(o => o.Metadata);
            }
            #endregion
            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                #region 更新报告相关流程状态及打回标记清除

                conn.UpdateOnly(report);
                #endregion

                #region 更新ReportItems流程状态
                //如果是受理至分派
                if (report.Status == (int)ReportStatusType.TaskAssign)
                {
                    foreach (var reportItem in reportItems)
                    {
                        if (reportItem.Status > (int)ReportItemStatusType.Asigning) continue;
                        reportItem.Status = (int)ReportItemStatusType.Asigning;
                        reportItem.AddOnlyProperties(o => o.Status);
                        //新增功能，若为受理至分派，当报告要求为数据时，清空所有标准值数据
                        if (!report.TestNeedJudge)
                        {
                            reportItem.DetectionValueQualified = string.Empty;
                            reportItem.AddOnlyProperties(o => o.DetectionValueQualified);
                        }
                        conn.UpdateOnly(reportItem);
                    }
                }
                //仅在任务分派下达时,更新ReportItem状态
                if (report.Status == (int)ReportStatusType.DataEntry)
                {
                    foreach (var reportItem in reportItems)
                    {
                        if (reportItem.Status > (int)ReportItemStatusType.Testing) continue;
                        reportItem.Status = (int)ReportItemStatusType.Testing;
                        reportItem.AddOnlyProperties(o => o.Status);
                        conn.UpdateOnly(reportItem);
                    }
                }
                //更新同步检验组信息（分派后）
                if (updateWorkGroupReportItems != null)
                {
                    foreach (var item in updateWorkGroupReportItems)
                    {
                        conn.UpdateOnly(item);
                    }
                }
                #endregion

                #region 流转记录相关信息创建(预留)，受理下达时创建ReportFiles
                report.AddReportNote("任务下达", AssignDescription((int)statusType), AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
                #endregion
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            report.RemoveAllOnlyProperties();
            return report;
        }

        /// <summary>
        /// 委托协议下达处理  400->项目检验 至 500->报告编制
        /// 参考 apps.jst-gov.com\Apps.Jst-Gov.Com.V3.5\Modules\Lims\Contracts\DataManager.aspx.cs 方法ReportFlowGo
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
        public static Report AssignTest(this Report report,ReportStatusType statusType, IDbConnection conn = null, bool useThread = false, bool isCompleteReport = true)
        {
            report.ThrowIfNull("请传入要下达的报告信息!");
            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            if (!isCompleteReport)
            {
                report = conn.SingleById<Report>(report.Kvid);
                report.ThrowIfNull("未找到对应的报告信息!");
            }
            //如果当前的报告已经不在项目检验了，不能对这份单子做任何的处理
            if (report.Status != (int)ReportStatusType.DataEntry) return report;


            //var queryReportItems = conn.From<ReportItem>().Where(o => o.ReportKvid == report.Kvid);
            //var allRreportItems = conn.Select(queryReportItems);
            var allRreportItems = report.GetOriginalReportItems(conn);
            allRreportItems.ThrowIfNullOrEmpty($"当前委托协议({report.ReportId})还未选择检验项目，请待选择检验项目后重新操作！");

            //#region 验证子检测项目数据结构完整性
            //var testKvids = allRreportItems.Select(o => o.Kvid).ToList();
            //(allRreportItems.Exists(o => o.ParentKvid != Guid.Empty && !Sql.In(o.ParentKvid, testKvids))).ThrowIfTrue("当前检验项目中有子项检验不对应大项检验的情况，如不是在委托受理阶段请联系系统管理员！");
            //#endregion

            //根据当前人的组别，只处理当前组的数据
            var primaryAllReportItems = allRreportItems.filterReportItem();
            if (statusType == ReportStatusType.CommissionAccept || statusType == ReportStatusType.TaskAssign)
            {
                if (!primaryAllReportItems.Exists(o => o.Status < int.MaxValue))
                {
                    report.Status = 500;
                    //新增：且要进行判断2021-10-21
                    if (report.TestNeedJudge)
                    {
                        if (primaryAllReportItems.Where(o => o.Judgement != null).ToList().Exists(o => o.Judgement.Contains("不合格")))
                        {
                            report.TestJudgement = "不合格";
                        }
                        else
                        {
                            report.TestJudgement = "合格";
                        }
                    }
                    else
                    {
                        report.TestJudgement = "/";
                    }
                    report.AddOnlyProperties(o => o.TestJudgement);
                    //若报告从数据录入打回至受理后，删除了未检测项目后，将直接下达到编制，此时需要更新检验完成日期
                    report.TestFinishDate = (report.TestFinishDate != null && report.TestFinishDate > (DateTime?)DateTime.MinValue) ? report.TestFinishDate : DateTime.Now;
                    report.AddOnlyProperties(o => o.TestFinishDate);
                }
                else
                {
                    report.Status = 400;
                }
                report.AddOnlyProperties(o => o.Status);

                #region 任务下达时，判断是否有打回信息，按需清除打回信息
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
                {
                    report.Metadata["CallBackTo"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
                {
                    report.Metadata["CallBackFrom"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                conn.UpdateOnly(report);
                report.AddReportNote("任务下达", AssignDescription(statusType == ReportStatusType.CommissionAccept ? 100 : 300), AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
            }
            else
            {
                IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
                if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
                try
                {
                    //当项目全部归档时，不执行下面更新项目状态的操作
                    if (primaryAllReportItems.Exists(o => o.Status != int.MaxValue))
                    {
                        var queryTestRoles = conn.From<EntityDuty<Test>>();
                        queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
                        queryTestRoles.Select(o => o.RoleName);
                        var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);

                        //所在组的全部大项
                        var ownerPrimaryItems = primaryAllReportItems.Where(o => o.WorkGroupKvid == KiviiContext.CurrentMember.DepartmentKvid|| Sql.In(o.WorkGroupName, testRoleNames)).ToList();

                        //没找到直接异常 
                        if (ownerPrimaryItems.IsNullOrEmpty())
                        {
                            if (report.StatusType == ReportStatusType.DataEntry) throw new Exception("当前组无项目需要下达！");
                            else
                            {
                                //记录原status
                                var originalStatus = report.Status;
                                report.StatusType = ReportStatusType.DataEntry;
                                report.AddOnlyProperties(o => o.Status);
                                conn.UpdateOnly(report);

                                report.AddReportNote("任务下达", AssignDescription(originalStatus), "项目检验", report.Metadata["LogDescription"], conn);

                                return report;
                            }


                        }
                        //记录当前组的组别，为创建项目的流转记录
                        var currentWorkGroupName = ownerPrimaryItems.FirstOrDefault().WorkGroupName;

                        var finalResults = ownerPrimaryItems;

                        var ownerNodeItems = allRreportItems.Where(o => Sql.In(o.ParentKvid, ownerPrimaryItems.ConvertAll(p => p.Kvid))).ToList();
                        if (!ownerNodeItems.IsNullOrEmpty()) finalResults.AddRange(ownerNodeItems);

                        var ownerParentItems = allRreportItems.Where(o => Sql.In(o.Kvid, ownerPrimaryItems.ConvertAll(p => p.ParentKvid))).ToList();

                        if (!ownerParentItems.IsNullOrEmpty())
                        {
                            finalResults.AddRange(ownerParentItems);

                            var ownerRootItems = allRreportItems.Where(o => Sql.In(o.Kvid, ownerParentItems.ConvertAll(p => p.ParentKvid))).ToList();
                            if (!ownerRootItems.IsNullOrEmpty()) finalResults.AddRange(ownerRootItems);
                        }
                        var reportItems = finalResults;
                        //获取未完成检验项目个数(只要没有判定结果的视为未检测)，加验证无实测值也要视为未检测
                        var unTested = reportItems.Where(o => o.Type == ReportItemType.Test && o.TestValue.IsNullOrEmpty()).Count();
                        (unTested > 0).ThrowIfTrue($"您还有【{unTested}】个检验任务未完成！");
                        var unJudgement = reportItems.Where(o => o.Type == ReportItemType.Test&&o.Judgement.IsNullOrEmpty()&&(!o.DetectionValueQualified.IsNullOrEmpty())).Count();
                        (unJudgement > 0).ThrowIfTrue($"您还有【{unJudgement}】个检验任务存在标准值，但未判定！");
                        //增加验证当子项全部判定后，若大项判定结果为空，异常提示
                        var testFolders = reportItems.Where(o => o.Type == ReportItemType.TestFolder).ToList();
                        foreach(var testFolder in testFolders)
                        {
                            var countNodeUnjudged = reportItems.Where(o=>o.ParentKvid == testFolder.Kvid&&o.Judgement.IsNullOrEmpty()).ToList().Count;
                            if (countNodeUnjudged == 0 && testFolder.Judgement.IsNullOrEmpty()) throw new Exception("子项均有判定结果，但大项未判定！无法下达报告！");
                        }
                        //先把Test的类型规范化是大项的全部设置类型为TestFolder,主要针对没有子项的大项类型也为Test
                        foreach (var item in reportItems)
                        {
                            if (item.Type != ReportItemType.Test) continue;
                            if (item.ParentKvid == Guid.Empty)
                            {
                                item.Type = ReportItemType.TestFolder;
                                item.Status = 111;
                            }
                            else
                            {
                                var parentTest = reportItems.FirstOrDefault(o => o.Kvid == item.ParentKvid);
                                if (parentTest != null && parentTest.Type != ReportItemType.TestFolder)
                                {
                                    item.Type = ReportItemType.TestFolder;
                                    item.Status = 111;
                                }
                            }
                        }
                        reportItems.RemoveAll(o => Sql.In(o.Type, ReportItemType.SampleSelf, ReportItemType.SamplePart, ReportItemType.Unsupported));
                        reportItems.ThrowIfNullOrEmpty($"当前委托协议({report.ReportId})还未选择检验项目，请待选择检验项目后重新操作！");

                        (reportItems.Exists(o => o.DetectionCodeActual.IsNullOrEmpty() && o.Type == ReportItemType.TestFolder)).ThrowIfTrue("您的检测项目(大项)含有没有测试方法的项目，无法下达！");

                        foreach (var item in reportItems)
                        {
                            if (item.Status == 111) item.Type = ReportItemType.Test;
                        }
                (reportItems.Exists(o => (o.TestReviewerName.IsNullOrEmpty() || o.TestReviewerKvid == Guid.Empty) && o.Type == ReportItemType.Test)).ThrowIfTrue("您有检测项目未进行复核，无法下达！");

                        //判断纤维含量的实测值相加是否为100，如果不是一百，不给下达,包括英文项目Fibre Content，实测值为/不加100验证
                        var fibrTests = reportItems.Where(o => (o.Title.Contains("纤维含量") || o.Title.Contains("Fibre Content")) && o.WorkGroupName == "纤维含量组" && !o.TestValue.IsNullOrEmpty() && o.TestValue != "/").ToList();
                        foreach (var fibre in fibrTests)
                        {
                            //2023/09/16刘娟需求，单组份（有100字眼）不需要公定
                            (fibre.TestValue.Contains("100") && fibre.Title.Contains("公定")).ThrowIfTrue("项目名称存在“公定”，请检查检测结果与项目名称是否匹配！");
                            //(!fibre.TestValue.Contains("100") && !fibre.Title.Contains("公定")).ThrowIfTrue("项目名称不存在“公定”，请检查检测结果与项目名称是否匹配！");
                            string result = fibre.TestValue.IsNumber();
                            string[] rtns = result.Split(new char[1] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                            decimal decl = 0;

                            foreach (var rtn in rtns)
                            {
                                if (string.IsNullOrEmpty(rtn)) continue;
                                decl += decimal.Parse(rtn);
                            }
                            (decl != 100 && decl % 100 != 0).ThrowIfTrue($"纤维含量的总和[{decl}!=100]数据不正确,请认真核对录入的数据信息！");
                            //2023-03-09刘娟需求，需比对以下实测值和测试方法
                            if (fibre.TestValue.Contains("山羊绒") || fibre.TestValue.Contains("马海毛") || fibre.TestValue.Contains("兔毛") || fibre.TestValue.Contains("驼毛") || fibre.TestValue.Contains("驼绒") || fibre.TestValue.Contains("牦牛毛") || fibre.TestValue.Contains("牦牛绒") || fibre.TestValue.Contains("羊驼毛") || fibre.TestValue.Contains("其他特种动物毛")) (fibre.DetectionCodeActual.Contains("16988-2013")|| fibre.DetectionCodeActual.Contains("16988—2013")).ThrowIfFalse("实测值和测试方法不匹配，请检查！");
                            if (fibre.TestValue.Contains("丙纶")|| fibre.TestValue.Contains("乙纶")) (fibre.DetectionCodeActual.Contains("01057.6-2007")|| fibre.DetectionCodeActual.Contains("01057.6—2007")).ThrowIfFalse("实测值和测试方法不匹配，请检查！");
                            if (fibre.TestValue.Contains("芳纶")) (fibre.DetectionCodeActual.Contains("01057.8-2012") || fibre.DetectionCodeActual.Contains("01057.8—2012")).ThrowIfFalse("实测值和测试方法不匹配，请检查！");
                        }
                        #region 更新ReportItem流程状态
                        var kvids = reportItems.Select(o => o.Kvid).ToList();
                        var updateReportItenLamda = conn.From<ReportItem>();
                        updateReportItenLamda = updateReportItenLamda.Update(o => o.Status);
                        updateReportItenLamda = updateReportItenLamda.Where(o => Sql.In(o.Kvid, kvids));
                        conn.UpdateOnly<ReportItem>(new ReportItem { Status = (int)ReportItemStatusType.TestCollected }, updateReportItenLamda);
                        //foreach (var reportItem in reportItems)
                        //{
                        //    reportItem.StatusType = ReportItemStatusType.TestCollected;
                        //    reportItem.AddOnlyProperties(o => o.Status);
                        //    conn.UpdateOnly(reportItem);
                        //}
                        //创建项目的流转记录
                        report.AddReportItemNote("项目下达", currentWorkGroupName, "检验完成", report.Metadata["LogDescription"], conn);
                        #endregion
                    }
                    int status = (int)ReportStatusType.ReportPreparation;
                    //判断是否所有的项目都是已归档状态，如果是，下达报告，否则只处理项目
                    #region 更新完后，重新查询，若所有项目都归档，则下达报告
                    var queryFinalReportItems = conn.From<ReportItem>();
                    queryFinalReportItems.Where(o => o.ReportKvid == report.Kvid);
                    var results = conn.Select(queryFinalReportItems);
                    var mainResults = results.filterReportItem();
                    if (!mainResults.Exists(o => o.Status != int.MaxValue))
                    {
                        #region 更新报告相关流程状态
                        //下达报告前，要重新查询下最新的报告状态，检查是否报告在数据录入状态
                        var latestReport = conn.SingleById<Report>(report.Kvid);
                        //2022年底编制需求，检验完成日期以实验室最后一次下达至编制为准；2024/03/04报告若无完成日期，下达到编制补充
                        report.TestFinishDate = (report.TestFinishDate != null && report.TestFinishDate > (DateTime?)DateTime.MinValue) ? report.TestFinishDate : DateTime.Now;
                        if (statusType == ReportStatusType.DataEntry) report.TestFinishDate = DateTime.Now;
                        report.AddOnlyProperties(o => o.TestFinishDate);
                        if (latestReport.StatusType == ReportStatusType.DataEntry)
                        {
                            report.Status = status;
                            report.AddOnlyProperties(o => o.Status);
                            
                            //记录报告初始的综合判定
                            var originalTestJudgement = report.TestJudgement;
                            if (report.TestNeedJudge)
                            {
                                //过滤判定结构为空或null的大项
                                var judgeResults = mainResults.Where(o => !o.Judgement.IsNullOrEmpty()).ToList();
                                if(judgeResults.IsNullOrEmpty()) report.TestJudgement = "/";
                                //报告要求为判定的，但是项目中无判定结果的，判定为/；
                                else if (!judgeResults.Exists(o=>o.Judgement.Contains("合格")))
                                {
                                    report.TestJudgement = "/";
                                }
                                else if (judgeResults.Exists(o => o.Judgement.Contains("不合格")))
                                {
                                    report.TestJudgement = "不合格";
                                }
                                else
                                {
                                    report.TestJudgement = "合格";
                                }
                            }
                            else
                            {
                                report.TestJudgement = "/";
                            }
                            report.AddOnlyProperties(o => o.TestJudgement);
                            #region 任务下达时，判断综合判定有无变化，若变化了，则需同时清空检验结论
                            if (!originalTestJudgement.IsNullOrEmpty()&& originalTestJudgement != report.TestJudgement)
                            {
                                report.Conclusion = string.Empty;
                                report.AddOnlyProperties(o => o.Conclusion);
                            }
                            #endregion
                            #region 任务下达时，判断是否有打回信息，按需清除打回信息
                            if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
                            {
                                report.Metadata["CallBackTo"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
                            {
                                report.Metadata["CallBackFrom"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            #endregion
                            #region 任务下达时，判断是否有直接返回信息，按需清除直接返回信息
                            if (report.Metadata != null && report.Metadata.ContainsKey("IsDirectReturn") && report.Metadata["IsDirectReturn"] == "Yes")
                            {
                                report.Metadata["IsDirectReturn"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            if (report.Metadata != null && report.Metadata.ContainsKey("DirectReturnStatus"))
                            {
                                report.Metadata["DirectReturnStatus"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            #endregion
                            #region 任务下达时，判断是否有ReviewingGroup信息，按需清除ReviewingGroup信息
                            if (report.Metadata != null && report.Metadata.ContainsKey("ReviewingGroup"))
                            {
                                report.Metadata["ReviewingGroup"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            #endregion
                            #region 任务下达时，判断是否有CallBackFromGroups信息，按需清除
                            if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFromGroups"))
                            {
                                report.Metadata["CallBackFromGroups"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            #endregion
                            #region 任务下达时，判断是否有IsCallbackAfterFinish信息，按需清除
                            if (report.Metadata != null && report.Metadata.ContainsKey("IsCallbackAfterFinish"))
                            {
                                report.Metadata["IsCallbackAfterFinish"] = null;
                                report.AddOnlyProperties(o => o.Metadata);
                            }
                            #endregion

                            conn.UpdateOnly(report);
                        }
                        #endregion

                        #region 流转记录相关信息创建(预留)
                        report.AddReportNote("任务下达", "项目检验", AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
                        #endregion
                    }
                    #endregion
                    trans?.Commit();
                }
                catch (Exception ex)
                {
                    trans?.Rollback();
                    throw ex;
                }

            }
            report.RemoveAllOnlyProperties();
            return report;
        }

        /// <summary>
        /// 委托协议下达处理  500->报告编制 至 600->报告审核
        /// 普通报告(W)无需审核过程,若调用此方法直接进入700->报告签发
        /// 参考 apps.jst-gov.com\Jst.Biz.Lims\Extensions\ReportExtension.cs 方法PreparationHandle
        /// 1、报告编制时需要验证编制人、审核人、签发人是否为同一人；
        /// 2、验证编制人的电子签名是否存在，如果不存在自动给出提示框提醒
        /// 3、生成报告检验报告的首页
        /// 4、更新所属当前报告的检验项目的进度Flow=600
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
        public static Report AssignPrepare(this Report report,ReportStatusType statusType, IDbConnection conn = null, bool isCompleteReport = true)
        {
            report.ThrowIfNull("请传入要下达的报告信息!");
            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            if (!isCompleteReport)
            {
                report = conn.SingleById<Report>(report.Kvid);
                report.ThrowIfNull("未找到对应的报告信息!");
            }
            //如果当前的报告已经不在报告编制了，不能对这份单子做任何的处理
            if (report.Status != (int)ReportStatusType.ReportPreparation) return report;
            //如果存在当前报告的检验项目的状态不在归档状态，则无法下达
            //var queryReportItems = conn.From<ReportItem>().Where(o => o.ReportKvid == report.Kvid);
            //var allRreportItems = conn.Select(queryReportItems);
            var allRreportItems = report.GetOriginalReportItems(conn);
            var allMainReportItems = allRreportItems.filterReportItem();
            if (allMainReportItems.Exists(o => o.Status != int.MaxValue)) throw new Exception("当前报告存在项目未完成，无法下达！");
            //如果下达时，无报告结论，禁止下达（避免误操作接口下达）
            if (report.Conclusion.IsNullOrEmpty()) throw new Exception("下达异常，请刷新后重试！");
            //如果下达时，无编制人，设置编制人为当前人
            //2021-10-12肖航需求，无论有无编制人，以最后下达为准
            report.PreparerKvid = KiviiContext.CurrentMember.Kvid;
            report.PreparerName = KiviiContext.CurrentMember.FullName;
            report.AddOnlyProperties(o => o.PreparerKvid);
            report.AddOnlyProperties(o => o.PreparerName);
            //(report.PreparerName.IsNullOrEmpty() || report.PreparerKvid == Guid.Empty).ThrowIfTrue("编制人不能为空或者匿名用户！");
            (!report.ReviewerName.IsNullOrEmpty() && report.PreparerName == report.ReviewerName).ThrowIfTrue("编制人不能和审核人相同！");
            (!report.IssuerName.IsNullOrEmpty() && report.PreparerName == report.IssuerName).ThrowIfTrue("编制人不能和签发人相同！");
            //**************************** 电子签名验证

            int status = (int)ReportStatusType.ReportReview;

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                #region 如果有直接返回的字段，需要清空相关信息
                if (report.Metadata.ContainsKey("IsDirectReturn") && report.Metadata["IsDirectReturn"] == "Yes")
                {
                    if (report.Metadata.ContainsKey("DirectReturnStatus"))
                    {
                        report.Metadata["DirectReturnStatus"] = null;
                    }
                    report.Metadata["IsDirectReturn"] = "No";
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                #region 任务下达时，判断是否有打回信息，按需清除打回信息
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
                {
                    report.Metadata["CallBackTo"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
                {
                    report.Metadata["CallBackFrom"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                #region 更新报告相关流程状态
                report.PreparationDate = DateTime.Now;//编制日期
                report.AddOnlyProperties(o => o.PreparationDate);
 
                //普通报告不审核 直接进入700->报告签发
                if (report.TypeEx.Contains("W")|| report.TypeEx.Contains("Z"))
                {
                    status = (int)ReportStatusType.ReportIssue;
                    report.ReviewDate = DateTime.Now;//审核日期
                    report.AddOnlyProperties(o => o.ReviewDate);
                }
                report.Status = status;
                report.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(report);
                #endregion

                #region 流转记录相关信息创建(预留)
                report.AddReportNote("任务下达", AssignDescription((int)statusType), AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
                #endregion
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            report.RemoveAllOnlyProperties();
            return report;
        }

        /// <summary>
        /// 委托协议下达处理  600->报告审核 至 700->报告签发
        /// 参考 apps.jst-gov.com\Jst.Biz.Lims\Extensions\ReportExtension.cs 方法ReviewHandle
        /// 1、报告审核时需要验证编制人、审核人、签发人是否为同一人；
        /// 2、验证编制人的电子签名是否存在，如果不存在自动给出提示框提醒
        /// 3、更新所属当前报告的检验项目的进度Flow=700
        /// 4、如果报告的首页以及报告的底页没有生成时需要及时的生成
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
        public static Report AssignCheck(this Report report, ReportStatusType statusType,IDbConnection conn = null, bool isCompleteReport = true)
        {
            report.ThrowIfNull("请传入要下达的报告信息!");
            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            if (!isCompleteReport)
            {
                report = conn.SingleById<Report>(report.Kvid);
                report.ThrowIfNull("未找到对应的报告信息!");
            }
            //如果当前的报告已经不在审核了，不能对这份单子做任何的处理
            if (report.Status != (int)ReportStatusType.ReportReview) return report;
            //如果存在当前报告的检验项目的状态不在归档状态，则无法下达
            //var queryReportItems = conn.From<ReportItem>().Where(o => o.ReportKvid == report.Kvid);
            //var allRreportItems = conn.Select(queryReportItems);
            var allRreportItems = report.GetOriginalReportItems(conn);
            var allMainReportItems = allRreportItems.filterReportItem();
            if (allMainReportItems.Exists(o => o.Status != int.MaxValue)) throw new Exception("当前报告存在项目未完成，无法下达！");
            //2021-10-12肖航需求，无论有无审核人，以最后下达为准，若是从委托受理直接至签发，不更新审核人
            //2022-06-23若是从编制直接至签发，不更新审核人
            if (statusType != ReportStatusType.CommissionAccept&& statusType != ReportStatusType.ReportPreparation)
            {
                report.ReviewerKvid = KiviiContext.CurrentMember.Kvid;
                report.ReviewerName = KiviiContext.CurrentMember.FullName;
                report.AddOnlyProperties(o => o.ReviewerKvid);
                report.AddOnlyProperties(o => o.ReviewerName);
            }
            
            //(report.PreparerName.IsNullOrEmpty() || report.PreparerKvid == Guid.Empty).ThrowIfTrue("编制人不能为空或者匿名用户！");
            (!report.PreparerName.IsNullOrEmpty() && report.ReviewerName == report.PreparerName).ThrowIfTrue("审核人不能和编制人相同！");
            (!report.IssuerName.IsNullOrEmpty() && report.ReviewerName == report.IssuerName).ThrowIfTrue("审核人不能和签发人相同！");
            //**************************** 电子签名验证

            int status = (int)ReportStatusType.ReportIssue;

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                #region 如果有直接返回的字段，需要清空相关信息
                if (report.Metadata.ContainsKey("IsDirectReturn") && report.Metadata["IsDirectReturn"] == "Yes")
                {
                    if (report.Metadata.ContainsKey("DirectReturnStatus"))
                    {
                        report.Metadata["DirectReturnStatus"] = "";
                    }
                    report.Metadata["IsDirectReturn"] = "No";
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                #region 任务下达时，判断是否有打回信息，按需清除打回信息
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
                {
                    report.Metadata["CallBackTo"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
                {
                    report.Metadata["CallBackFrom"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                #region 更新报告相关流程状态
                report.ReviewDate = DateTime.Now;//审核日期
                report.AddOnlyProperties(o => o.ReviewDate);
                report.Status = status;
                report.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(report);
                #endregion

                #region 流转记录相关信息创建(预留)
                report.AddReportNote("任务下达", AssignDescription((int)statusType), AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
                #endregion

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            report.RemoveAllOnlyProperties();
            return report;
        }

        /// <summary>
        /// 委托协议下达处理  700->报告签发 至 800->报告打印
        /// 参考 apps.jst-gov.com\Jst.Biz.Lims\Extensions\ReportExtension.cs 方法IssuanceHandle
        /// 1、如果下达时没有报告首页要及时给出提醒并且返回
        /// 2、报告审核时需要验证编制人、审核人、签发人是否为同一人；
        /// 3、验证编制人的电子签名是否存在，如果不存在自动给出提示框提醒
        /// 4、更新所属当前报告的检验项目的进度Flow=800(备注：如果报告份数为0的，报告及时归档Flow=int.MaxValue)
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
        public static Report AssignIssue(this Report report, ReportStatusType statusType,DateTime? issueDate = null,IDbConnection conn = null, bool isCompleteReport = true)
        {
            report.ThrowIfNull("请传入要下达的报告信息!");
            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            if (!isCompleteReport)
            {
                report = conn.SingleById<Report>(report.Kvid);
                report.ThrowIfNull("未找到对应的报告信息!");
            }
            //如果当前的报告已经不再签发了，直接返回
            if (report.Status != (int)ReportStatusType.ReportIssue) return report;
            //如果存在当前报告的检验项目的状态不在归档状态，则无法下达
            //var queryReportItems = conn.From<ReportItem>().Where(o => o.ReportKvid == report.Kvid);
            //var allRreportItems = conn.Select(queryReportItems);
            var allRreportItems = report.GetOriginalReportItems(conn);
            var allMainReportItems = allRreportItems.filterReportItem();
            if (allMainReportItems.Exists(o => o.Status != int.MaxValue)) throw new Exception("当前报告存在项目未完成，无法下达！");
            //如果下达时，无签发人，设置签发人为当前人
            //2023-03-13黄松需求，无论有无签发人，以最后下达为准
            report.IssuerKvid = KiviiContext.CurrentMember.Kvid;
            report.IssuerName = KiviiContext.CurrentMember.FullName;
            report.AddOnlyProperties(o => o.IssuerKvid);
            report.AddOnlyProperties(o => o.IssuerName);
            
            //**************************** 电子签名验证
            //**************************** 判断生成的报告是否存在(包括首页、附页及归档的报告)
            //(report.IssuerName.IsNullOrEmpty() || report.IssuerKvid == Guid.Empty).ThrowIfTrue("签发人不能为空或匿名用户！");
            (!report.PreparerName.IsNullOrEmpty() && report.IssuerName == report.PreparerName).ThrowIfTrue("编制人不能和签发人相同！");
            (!report.ReviewerName.IsNullOrEmpty() && report.IssuerName == report.ReviewerName).ThrowIfTrue("审核人不能和签发人相同！");

            List<Guid> guids = new List<Guid>();
            guids.Add(report.PreparerKvid);
            guids.Add(report.ReviewerKvid);
            guids.Add(report.IssuerKvid);
            guids.Remove(Guid.Empty);
            var discount = guids.Distinct().Count();
            //int tmp = 3;
            //if (report.Type.Contains("W")) tmp = 2;
            //(guids.Count != discount || discount < tmp).ThrowIfTrue("编制、审核、签发人不能有重复！");

            //根据报告份数，来判断是否进入打印流程
            int status = report.Quantity == 0 ? int.MaxValue : (int)ReportStatusType.ReportPrint;

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);         
            try
            {
                #region 往打印池推送待打印报告记录
                if (status == (int)ReportStatusType.ReportPrint)
                {
                    //
                }
                #endregion

                #region 任务下达时，判断是否有打回信息，按需清除打回信息
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
                {
                    report.Metadata["CallBackTo"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
                {
                    report.Metadata["CallBackFrom"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                //清除系统老数据的flowcallback
                if (report.Metadata != null && report.Metadata.ContainsKey("FlowCallBack"))
                {
                    report.Metadata["FlowCallBack"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                #region 更新报告相关流程状态
                //签发下达后清除所有打回及相关标记
                if (report.Metadata.ContainsKey("IsDirectReturn")) report.Metadata["IsDirectReturn"] = null;
                if (report.Metadata.ContainsKey("DirectReturnStatus")) report.Metadata["DirectReturnStatus"] = null;
                //若传入签发日期参数，则以传入参数为签发日期
                report.AddOnlyProperties(o => o.Metadata);
                report.IssueDate = DateTime.Now;
                if (issueDate != null)
                {
                    report.IssueDate = issueDate;
                }
                report.AddOnlyProperties(o => o.IssueDate);
                report.Status = status;
                report.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(report);
                #endregion

                #region 流转记录相关信息创建(预留)
                report.AddReportNote("任务下达", AssignDescription((int)statusType), AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
                #endregion

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            report.RemoveAllOnlyProperties();
            return report;
        }

        /// <summary>
        /// 委托协议下达处理  800->报告打印 至 int.MaxValue->报告归档
        /// 参考 apps.jst-gov.com\Apps.Jst-Gov.Com.V3.5\Modules\Lims\Reports\Themes\Printer-List.ascx.cs 方法btnDelete_Click
        /// 4、更新所属当前报告的检验项目的进度Flow=int.MaxValue
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
        public static Report AssignPrint(this Report report,ReportStatusType statusType, IDbConnection conn = null, bool isCompleteReport = true)
        {
            report.ThrowIfNull("请传入要下达的报告信息!");
            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            if (!isCompleteReport)
            {
                report = conn.SingleById<Report>(report.Kvid);
                report.ThrowIfNull("未找到对应的报告信息!");
            }
            if (report.Status != (int)ReportStatusType.ReportPrint) return report;
            //如果存在当前报告的检验项目的状态不在归档状态，则无法下达
            //var queryReportItems = conn.From<ReportItem>().Where(o => o.ReportKvid == report.Kvid);
            //var allRreportItems = conn.Select(queryReportItems);
            var allRreportItems = report.GetOriginalReportItems(conn);
            var allMainReportItems = allRreportItems.filterReportItem();
            if (allMainReportItems.Exists(o => o.Status != int.MaxValue)) throw new Exception("当前报告存在项目未完成，无法下达！");

            int status = int.MaxValue;

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                #region 任务下达时，判断是否有打回信息，按需清除打回信息
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackTo") && report.Metadata["CallBackTo"] == statusType.ToString())
                {
                    report.Metadata["CallBackTo"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                if (report.Metadata != null && report.Metadata.ContainsKey("CallBackFrom") && report.Metadata["CallBackFrom"] == statusType.ToString())
                {
                    report.Metadata["CallBackFrom"] = null;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                #endregion

                #region 更新报告相关流程状态
                report.Status = status;
                report.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(report);
                #endregion

                #region 流转记录相关信息创建(预留)
                report.AddReportNote("任务下达", AssignDescription((int)statusType), AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
                #endregion

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            report.RemoveAllOnlyProperties();
            return report;
        }

        /// <summary>
        /// 委托协议下达处理  999->管理部审核 至 700->报告签发
        /// 移除
        /// </summary>
        /// <param name="report">报告的进度Status 100->委托受理、300->任务分派、400->项目检验、500->报告编制、600->报告审核、700->报告签发、800->报告打印、int.MaxValue->报告归档</param>
        /// <param name="conn"></param>
        /// <param name="isCompleteReport">是否是完整的报告实体,默认是,若不是则会重新查询数据库</param>
        /// <returns></returns>
//        public static Report AssignManager(this Report report, IDbConnection conn = null, bool isCompleteReport = true)
//        {
//            report.ThrowIfNull("请传入要下达的报告信息!");
//            report.Kvid.ThrowIfEmpty("报告Kvid不能为空!");

//            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
//            if (conn == null)
//            {
//                conn = KiviiContext.GetOpenedDbConnection<Report>();
//            }
//            if (!isCompleteReport)
//            {
//                report = conn.SingleById<Report>(report.Kvid);
//                report.ThrowIfNull("未找到对应的报告信息!");
//            }
//            if (report.Status != 999) return report;


//            int status = (int)ReportStatusType.ReportIssue;

//            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
//#if !CLOSETRANS
//            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
//#endif
//            try
//            {
//#region 更新报告相关流程状态
//                report.Status = status;
//                report.AddOnlyProperties(o => o.Status);
//                report.ReviewerName = KiviiContext.CurrentMember.FullName;
//                report.ReviewerKvid = KiviiContext.CurrentMember.Kvid;
//                report.AddOnlyProperties(o => o.ReviewerName);
//                report.AddOnlyProperties(o => o.ReviewerKvid);
//                conn.UpdateOnly(report);
//#endregion

//#region 流转记录相关信息创建(预留)
//                report.AddReportNote("下达", "管理部审核", AssignDescription(report.Status), report.Metadata["LogDescription"], conn);
//                //if (status == int.MaxValue)
//                //{
//                //    report.AddReportNote("下达", "报告打印", "报告归档", "", conn);
//                //}
//#endregion
//                trans?.Commit();
//            }
//            catch (Exception ex)
//            {
//                trans?.Rollback();
//                throw ex;
//            }

//            report.RemoveAllOnlyProperties();
//            return report;
//        }
        #endregion

        /// <summary>
        /// 根据报告KVID和ReportFile记录生成文件
        /// 报告生成V2.0
        /// 流程接口调用适配
        /// </summary>
        /// <param name="report"></param>
        /// <param name="savePath"></param>
        /// <param name="originalStatus"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static bool GenerateReport(this Report report,  string savePath, int originalStatus, bool useThread = true,IDbConnection conn = null)
        {
            savePath.ThrowIfNullOrEmpty("参数非法：SavaPath不能为空！");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();

            //生成该报告对应模板的ReportFile文件，无实际excel记录
            if (report.TemplateKvid == Guid.Empty) throw new Exception("当前报告未选择报告模板！无法下达！");

            var queryReportFile = conn.From<ReportFile>();
            queryReportFile.Where(o => o.ReportKvid == report.Kvid && o.TemplateKvid == report.TemplateKvid);
            var rtnReportFiles = conn.Select(queryReportFile);
            if (rtnReportFiles.Count == 0)
            {
                var queryTemplate = conn.From<Template>();
                queryTemplate.Where(o => o.Kvid == report.TemplateKvid);
                var rtnTemplate = conn.Select(queryTemplate).FirstOrDefault();
                rtnTemplate.ThrowIfNull("未找到当前报告所对应模板！");

                var queryTemplateFile = conn.From<TemplateFile>();
                queryTemplateFile.Where(o => o.TemplateKvid == rtnTemplate.Kvid && o.Tags != "" && o.Tags != null);
                var rtnTemplateFile = conn.Select(queryTemplateFile);
                rtnTemplateFile.ThrowIfNull("当前报告所关联的模板无文件！");

                foreach (var en in rtnTemplateFile)
                {
                    var reportFile = new ReportFile();
                    reportFile.ReportKvid = report.Kvid;
                    reportFile.TemplateKvid = en.TemplateKvid;
                    reportFile.TemplateFileKvid = en.Kvid;
                    reportFile.Name = en.Name;
                    reportFile.Tags = en.Tags;
                    reportFile.TemplateFilePath = en.Path;
                    reportFile.SortID = en.SortId;
                    conn.Insert(reportFile);
                    reportFile.RemoveAllOnlyProperties();
                    rtnReportFiles.Add(reportFile);
                }
            }

            foreach (var reportFile in rtnReportFiles)
            {
                var tagList = reportFile.Tags.getTagList();
                foreach (var en in tagList)
                {
                    if (en.Key == originalStatus)
                    {
                        string openPath = "";
                        if (en.Value == "G"|| reportFile.Path.IsNullOrEmpty())//重新生成。或者reportFile.Path为空
                        {
                            if (reportFile.TemplateFilePath != null && reportFile.TemplateFilePath != "")
                            {
                                openPath = reportFile.TemplateFilePath;
                            }
                            else
                            {
                                var templateFile = conn.SingleById<TemplateFile>(reportFile.TemplateFileKvid);
                                templateFile.ThrowIfNull("未找到关联的模板文件，无法重新生成！");
                                openPath = templateFile.Path;
                            }
                        }
                        else//在原有的报告文件基础上生成
                        {
                            openPath = reportFile.Path;
                        }
                        var sPath = $"/Storages{savePath}/{report.Kvid}/{report.ReportId}-{reportFile.Name}";
                        report.excuteGenerate(openPath, sPath, useThread);
                        reportFile.Path = sPath;
                        reportFile.AddOnlyProperties(o => o.Path);
                        conn.UpdateOnly(reportFile);
                        break;
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// 根据以逗号分隔的字符串，返回字典对象
        /// </summary>
        /// <param name="tags"></param>
        /// <returns></returns>
        public static Dictionary<int, string> getTagList(this string tags)
        {
            tags = tags.Replace("，", ",");
            List<string> str = new List<string>();
            str = tags.Split(',').ToList();
            var strObj = new Dictionary<int, string>();

            foreach (var en in str)
            {
                int numbers = int.Parse(Regex.Replace(en, "[a-z]", "", RegexOptions.IgnoreCase));
                string strs = Regex.Replace(en, @"[0-9]+", "");
                strObj.Add(numbers, strs);
            }
            return strObj;
        }

        /// <summary>
        /// 执行生成报告
        /// </summary>
        /// <param name="report"></param>
        /// <param name="openPath"></param>
        /// <param name="savePath"></param>
        public static void excuteGenerate(this Report report,string openPath,string savePath,bool useThread = true)
        {
            if (useThread)
            {
                var th = KiviiContext.NewThread($"生成EXCEL:{savePath}",() => {

                    KiviiContext.Logger.Debug($"Generate Report {report.ReportId} In Thread");
                    excuteGenerateMethod(report, openPath, savePath);
                    KiviiContext.Logger.Debug($"Generate Report {report.ReportId} Completed");

                });
                th.Start();
            }
            else
            {
                excuteGenerateMethod(report, openPath, savePath);
            }
            
        }
        /// <summary>
        /// 原来的执行方法,如果是同步的话，可以直接调用这个方法，异步调用上面的方法
        /// </summary>
        /// <param name="report"></param>
        /// <param name="openPath"></param>
        /// <param name="savePath"></param>
        public static void excuteGenerateMethod(this Report report, string openPath, string savePath)
        {
            //var tasks = report.GetPlatReportItems();
            var sources = report.GetSources();
            var templateWorkbook = new TemplateWorkbook();
            var preparerSignature = "/storages/Organizations/Member/Signatures/" + report.PreparerKvid + "/" + report.PreparerName + ".*";
            var reviewerSignature = "/storages/Organizations/Member/Signatures/" + report.ReviewerKvid + "/" + report.ReviewerName + ".*";
            var issuerSignature = "/storages/Organizations/Member/Signatures/" + report.IssuerKvid + "/" + report.IssuerName + ".*";
            templateWorkbook.Open(openPath);
            templateWorkbook.Bind("contract", report);
            templateWorkbook.Bind("Model", report);
            //templateWorkbook.Bind("task", tasks);
            templateWorkbook.Bind("imageList", sources);
            templateWorkbook.Bind("prepareSignature", preparerSignature);
            templateWorkbook.Bind("reviewerSignature", reviewerSignature);
            templateWorkbook.Bind("issuerSignature", issuerSignature);
            var res = templateWorkbook.Execute();
            res = res.Where(o => o.Type != TemplateExecuteResultType.Succeed).ToList();

            var mStream = new MemoryStream();
            templateWorkbook.SaveAs(mStream);

            var virtualFile = KiviiContext.VirtualFiles.WriteFile(savePath, mStream);
        }
        /// <summary>
        /// 执行合并报告
        /// </summary>
        /// <param name="report"></param>
        /// <param name="savePath"></param>
        /// <param name="conn"></param>
        public static void excuteCombine(this Report report,string savePath, IDbConnection conn = null)
        {
            savePath.ThrowIfNullOrEmpty("参数非法：SavaPath不能为空！");
            var sPath = $"/Storages{savePath}/{report.Kvid}/{report.ReportId}.xlsx";
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            report.TemplateKvid.ThrowIfEmpty("当前报告未关联模板，无法生成！");

            var queryReportFile = conn.From<ReportFile>();
            queryReportFile.Where(o => o.ReportKvid == report.Kvid && o.TemplateKvid == report.TemplateKvid);
            queryReportFile.OrderBy(o => o.SortID);
            var rtnReportFiles = conn.Select(queryReportFile).ToList();
            rtnReportFiles.ThrowIfNull("当前报告的ReportFile为空，无法生成！");

            if (rtnReportFiles.Exists(o => o.Path.IsNullOrEmpty())) throw new Exception("存在Path为空的ReportFile记录，请先生成好对应文件");
            //模板必须先open，才能Contact，所以先打开待合并的第一个文件，然后向后面合并；
            var firstPath = rtnReportFiles.FirstOrDefault().Path;
            rtnReportFiles.RemoveAt(0);
            
            
            //TemplateWorkbook.CatchFileStreamFn=KiviiContext.c
            var templateWorkbook = new TemplateWorkbook();
            templateWorkbook.Open(firstPath);
            if (rtnReportFiles.Count > 0)//只有ReportFile数量大于0才需要合并
            {
                foreach (var en in rtnReportFiles)
                {
                    en.Path = KiviiContext.VirtualFiles.GetFile(en.Path).RealPath;
                }
                string[] contactArrays = rtnReportFiles.Select(o => o.Path).ToArray();
                templateWorkbook.Contact(contactArrays);
            }
 
            //var res = templateWorkbook.Execute();
            //res = res.Where(o => o.Type != TemplateExecuteResultType.Succeed).ToList();
            var mStream = new MemoryStream();
            templateWorkbook.SaveAs(mStream);
            KiviiContext.VirtualFiles.WriteFile(sPath, mStream);
        }

        /// <summary>
        /// 报告下达前，根据检验项目的某些信息判定该报告下达到哪个节点
        /// 100=>委托受理，200=>合同评审，300=>任务分派，400=>项目检验，500=>报告编制
        /// 无论是从哪个节点打回的报告，都必须经过编制
        /// </summary>
        /// <param name="report"></param>
        /// <param name="conn"></param>
        public static void beforeAsign(this Report report, ReportStatusType statusType, IDbConnection conn = null)
        {
            //if (report.Status >= 400) return;
            //如果是数据录入点下达，不考虑直接返回
            if (statusType == ReportStatusType.DataEntry)
            {
                report.Status = (int)ReportStatusType.DataEntry;
                return;
            }
            //如果是直接返回
            if (report.Metadata.ContainsKey("IsDirectReturn") && report.Metadata["IsDirectReturn"] == "Yes")
            {
                if (report.Metadata.ContainsKey("DirectReturnStatus"))
                {
                    report.Status = int.Parse(report.Metadata["DirectReturnStatus"]) - 100;
                    //如果是从归档直接到受理，将报告下达到签发
                    if (report.Status > 800)
                    {
                        report.Status = 600;
                    }
                    return;
                }
            }
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            var rtnReportItem = report.GetOriginalReportItems(conn);
            report.Tag = rtnReportItem;
            var mainReportItems = rtnReportItem.filterReportItem();
            if (mainReportItems.Exists(o => o.StatusType == ReportItemStatusType.BeforeTest))
            {
                report.Status = (int)ReportStatusType.CommissionAccept;
                return;
            }
            if (report.TaskAssignDate == null) return;
            if (mainReportItems.Exists(o => o.WorkGroupKvid == Guid.Empty))
            {
                report.Status = (int)ReportStatusType.CommissionAccept;
                return;
            }
            if ((report.Status == (int)ReportStatusType.ReportPreparation || report.Status == (int)ReportStatusType.ReportReview || report.Status == (int)ReportStatusType.ReportIssue || report.Status == (int)ReportStatusType.ReportPrint) && !mainReportItems.Exists(o => o.Status < int.MaxValue))
            {
                return;
            }
            //处理更改报告
            if (report.BizType == "X") return;
            if (report.BizType == "G0") return;
            //如果是数据录入下达，直接走数据录入下达
            if (report.Status == (int)ReportStatusType.DataEntry) return;

            //处理打回报告下达后
            if (mainReportItems.Exists(o => o.Status < (int)ReportStatusType.TaskAssign)) report.Status = (int)ReportStatusType.TaskAssign;
            else report.Status = (int)ReportStatusType.DataEntry;
        }

        public static void excuteGenerateX(this Report report, IDbConnection conn = null)
        {
            //根据-x报告找到原报告的附页，在原报告的基础上生成附页
            //其实就是重新生成条码和二维码。其他不变
            report.ThrowIfNull("报告为空");
            //report.ReportId.ThrowIfNull("报告编号为空");
            //var index = report.ReportId.LastIndexOf("-X");
            //兼容新版更改报告
            //if (index < 0) index = report.ReportId.LastIndexOf("-G");
            //var originalReportId = report.ReportId.Substring(0, index);
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            //var queryOriginalReportReport = conn.From<Report>();
            //queryOriginalReportReport.Where(o => o.ReportId == originalReportId);
            //var originalReport = conn.Select(queryOriginalReportReport).FirstOrDefault();
            var originalReport = conn.SingleById<Report>(report.BizKvid);
            originalReport.ThrowIfNull("未找到原报告！");

            //查原报告的附页文件
            //2024.4.11修复，应取最新的报告的附页进行生成
            var connFile = KiviiContext.GetOpenedDbConnection<EntityDbFile<EntityGeneric>>();
            var queryFile = connFile.From<EntityDbFile<EntityGeneric>>();
            queryFile.Where(o => o.OwnerKvid == originalReport.Kvid&&o.Name.Contains("附页"));
            var file = connFile.Select(queryFile).FirstOrDefault();
            file.ThrowIfNull("原报告的附页未找到，请联系管理员重新生成");
            var openPath = $"/Storages{file.DbFolderPath}/{file.Kvid}/{file.Name}";
            var savePath = "/Report/Excel";
            //savePath应保持与openPath一致，暂且写死测试

            
            var sPath = $"/Storages{savePath}/{report.Kvid}/{report.ReportId}-报告附页{file.Extension}";

            //更新-X报告的附页ReportFile的Path值，用来后面签发合成excel
            var queryReportFile = conn.From<ReportFile>();
            queryReportFile.Where(o => o.ReportKvid == report.Kvid && o.Name.Contains("附页")&&o.TemplateKvid == report.TemplateKvid);

            var reportFile = conn.Select(queryReportFile).FirstOrDefault();
            reportFile.ThrowIfNull("未找到对应的附页");
            reportFile.Path = sPath;
            reportFile.AddOnlyProperties(o => o.Path);
            conn.UpdateOnly(reportFile);

            var templateWorkbook = new TemplateWorkbook();
            templateWorkbook.Open(openPath);
            templateWorkbook.Bind("contract", report);
            templateWorkbook.Execute();

            var mStream = new MemoryStream();
            templateWorkbook.SaveAs(mStream);

            var virtualFile = KiviiContext.VirtualFiles.WriteFile(sPath, mStream);

            return;
        }

        /// <summary>
        /// 获取当前登陆人的角色权限
        /// 例如：张三，角色权限涉及：安全组、物理组。返回组信息
        /// </summary>
        /// <param name="currentMemberKvid"></param>
        /// <param name="reportItems"></param>
        /// <returns></returns>
        public static List<string> GetTestRoles(this Guid currentMemberKvid)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EntityDuty<Test>>();
            var queryTestRoles = conn.From<EntityDuty<Test>>();
            queryTestRoles.Where(o => o.OwnerKvid == currentMemberKvid);
            queryTestRoles.GroupBy(o => o.RoleName);
            queryTestRoles.Select(o => o.RoleName);
            var roles = conn.Select<string>(queryTestRoles);
            if (roles.IsNullOrEmpty()) return null;
            else return roles;
        }
        ///// <summary>
        ///// 计算报告检验依据的方法，根据参数计算报告的返回字段中是否要计算以下的结果
        ///// </summary>
        ///// <param name="report"></param>
        ///// <param name="calcForceStandard">是否要计算强制标准项目</param>
        ///// <param name="calcProductStandard">是否要计算产品标准项目</param>
        ///// <param name="calcProductStandardWithoutForce">是否要计算产品标准项目，排除强制标准</param>
        ///// <param name="calcMethodStandard">是否要计算方法标准</param>
        ///// <param name="conn"></param>
        ///// <returns></returns>
        //public static Report CalculateStandard(this Report report,bool combineProductAndMethod,bool calcForceStandard = false,List<string> forceStandards = null,bool calcProductStandard = false,bool calcProductStandardWithoutForce = false,bool calcMethodStandard = false, IDbConnection conn = null)
        //{
        //    //参数calcForceStandard为true，必须传入强标list
        //    if (calcForceStandard && forceStandards == null) throw new Exception("参数异常，未传入强制标准列表");
        //    //获取报告
        //    (report.Kvid == Guid.Empty).ThrowIfTrue("缺少ReportKvid无法同步!");
        //    report.ThrowIfNull("未找到需要同步的报告!");
        //    bool disposeConn = conn == null;
        //    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
        //    //获取ReportItem
        //    var queryReportItems = conn.From<ReportItem>();
        //    queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx);

        //    //整个报告的全部项目
        //    var allReportItems = conn.Select(queryReportItems);
        //    if (disposeConn) conn.Dispose();
        //    //全项中的大项
        //    var primaryAllReportItems = allReportItems.filterReportItem();
        //    if (primaryAllReportItems.IsNullOrEmpty())
        //    {
        //        report.TestStandards = "";
        //        report.AddOnlyProperties(o => o.TestStandards);
        //        return report;
        //    }

        //    //检验依据,按需计算
        //    var testStandards = string.Empty;
        //    var testStandardsList = new List<string>();

        //    var testStandardsForce = string.Empty;
        //    var testStandardsForceList = new List<string>();

        //    var testStandardsProduct = string.Empty;
        //    var testStandardsProductList = new List<string>();

        //    var testStandardsProductWithoutForce = string.Empty;
        //    var testStandardsProductWithoutForceList = new List<string>();

        //    var testStandardsMethod = string.Empty;
        //    var testStandardsMethodList = new List<string>();

        //    #region 组合检验依据

        //    //遍历所有的大项，进行数据组合
        //    foreach(var item in primaryAllReportItems)
        //    {
        //        //组合产品和方法返回
        //        testStandardsList.Add(item.StandardCode+(item.StandardTitle.IsNullOrEmpty()?"":"《"+item.StandardTitle+"》"));
        //        //判定强制标准
        //        if (forceStandards != null)
        //        {
        //            foreach (var std in forceStandards)
        //            {
        //                if (item.StandardCode.Contains(std)) testStandardsForceList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
        //            }
        //        }
        //        //方法标准
        //        if (item.DetectionReferenceKvid == Guid.Empty)
        //        {
        //            testStandardsMethodList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
        //        }
        //        //产品标准
        //        else
        //        {
        //            testStandardsProductList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
        //            if (forceStandards != null)
        //            {
        //                foreach (var std in forceStandards)
        //                {
        //                    if (!item.StandardCode.Contains(std)) testStandardsProductWithoutForceList.Add(item.StandardCode + (item.StandardTitle.IsNullOrEmpty() ? "" : "《" + item.StandardTitle + "》"));
        //                }
        //            }
        //        }
        //    }
        //    testStandardsList = testStandardsList.Distinct().ToList();
        //    testStandards = String.Join("、", testStandardsList);

        //    testStandardsProductList = testStandardsProductList.Distinct().ToList();
        //    testStandardsProduct = String.Join("、", testStandardsProductList);

        //    testStandardsProductWithoutForceList = testStandardsProductWithoutForceList.Distinct().ToList();
        //    testStandardsProductWithoutForce = String.Join("、", testStandardsProductWithoutForceList);

        //    testStandardsMethodList = testStandardsMethodList.Distinct().ToList();
        //    testStandardsMethod = String.Join("、", testStandardsMethodList);

        //    testStandardsForceList = testStandardsForceList.Distinct().ToList();
        //    testStandardsForce = String.Join("、", testStandardsForceList);

        //    //根据传入参数，组织返回数据，只有testStandardsProduct、testStandards、testStandardsProductWithoutForce使用正式字段，其他使用metadata字段
        //    if (combineProductAndMethod)
        //    {
        //        report.TestStandards = testStandards;
        //    }
        //    else if (calcProductStandard)
        //    {
        //        report.TestStandards = testStandardsProduct;
        //    }
        //    else if (calcProductStandardWithoutForce)
        //    {
        //        report.TestStandards = testStandardsProductWithoutForce;
        //    }
        //    else
        //    {
        //        report.TestStandards = string.Empty;
        //    }
        //    if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
        //    //方法标准
        //    if(calcMethodStandard)
        //    {
        //        report.Metadata["testStandardsMethod"] = testStandardsMethod;
        //        report.AddOnlyProperties(o => o.Metadata);
        //    }
        //    //国标
        //    if (calcForceStandard)
        //    {
        //        report.Metadata["testStandardsForce"] = testStandardsForce;
        //        report.AddOnlyProperties(o => o.Metadata);
        //    }
        //    report.AddOnlyProperties(o => o.TestStandards);
        //    #endregion
        //    return report;
        //}

        ///// <summary>
        ///// 组合检验项目方法
        ///// </summary>
        ///// <param name="report"></param>
        ///// <param name="conn"></param>
        ///// <returns></returns>
        //public static Report CalculateReportDetection(this Report report, IDbConnection conn = null)
        //{
        //    //获取报告
        //    (report.Kvid == Guid.Empty).ThrowIfTrue("缺少ReportKvid无法同步检验标准及项目!");
        //    report.ThrowIfNull("未找到需要同步的报告!");
        //    bool disposeConn = conn == null;
        //    if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
        //    //获取ReportItem
        //    var queryReportItems = conn.From<ReportItem>();
        //    queryReportItems.Where(o => o.ReportKvid == report.Kvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx);

        //    //整个报告的全部项目
        //    var allReportItems = conn.Select(queryReportItems);
        //    if (disposeConn) conn.Dispose();
        //    //全项中的大项
        //    var primaryAllReportItems = allReportItems.filterReportItem();
        //    if (primaryAllReportItems.IsNullOrEmpty())
        //    {
        //        report.TestStandards = "";
        //        report.AddOnlyProperties(o => o.TestStandards);
        //        report.TestDetections = "";
        //        report.AddOnlyProperties(o => o.TestDetections);
        //        return report;
        //    }
        //    //primaryAllReportItems.ThrowIfNullOrEmpty("此报告全部检验项目未找到检验大项,结构异常,请联系管理员!");

        //    //查看是否已经存在色号
        //    bool existSelf = allReportItems.Exists(o => o.Type == ReportItemType.SampleSelf);

        //    var testDetections = string.Empty;//检验项目
        //    var testDetectionsList = new List<string>();

        //    #region 组合检验项目
        //    //根据大项进行分组
        //    var groupPrimarys = primaryAllReportItems.GroupBy(o => o.Title);
        //    foreach (var kv in groupPrimarys)
        //    {
        //        var detectionStr = kv.Key;

        //        var self = new List<string>();
        //        var part = new List<string>();
        //        var actual = new List<string>();
        //        var method = new List<string>();

        //        //存在色号的情况
        //        if (existSelf)
        //        {
        //            //key色号,value部位集合
        //            var selfAndParts = new Dictionary<string, List<string>>();
        //            foreach (var primary in kv)
        //            {
        //                var parentItem = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid);
        //                //存在色号情况下 不可能为空  为空即跳过不处理
        //                if (parentItem == null) continue;

        //                var selfParent = parentItem;
        //                if (parentItem.Type == ReportItemType.SamplePart) selfParent = allReportItems.FirstOrDefault(o => o.Kvid == parentItem.ParentKvid);

        //                if (selfParent == null) continue;

        //                var detectionMethodDescription = "";
        //                if (primary.Metadata.ContainsKey("DetectionMethodDescription"))
        //                {
        //                    detectionMethodDescription = primary.Metadata["DetectionMethodDescription"].IsNullOrEmpty() ? "" : primary.Metadata["DetectionMethodDescription"];
        //                }
        //                var detectionCodeActual = primary.DetectionCodeActual;
        //                if (primary.Metadata.ContainsKey("DetectionCodeActual")) detectionCodeActual = primary.Metadata["DetectionCodeActual"];
        //                if (!detectionMethodDescription.IsNullOrEmpty())
        //                {
        //                    detectionCodeActual = detectionCodeActual.Replace(detectionMethodDescription, "");
        //                    detectionMethodDescription = " " + detectionMethodDescription;
        //                }
        //                if (!actual.Contains(primary.DetectionCodeActual)) actual.Add(detectionCodeActual);

        //                if (!method.Contains(detectionMethodDescription)) method.Add(detectionMethodDescription);

        //                //说明有部位
        //                if (parentItem.Type == ReportItemType.SamplePart)
        //                {
        //                    if (selfAndParts.ContainsKey(selfParent.Title))
        //                    {
        //                        if (!selfAndParts[selfParent.Title].Contains(parentItem.Title)) selfAndParts[selfParent.Title].Add(parentItem.Title);
        //                    }
        //                    else
        //                    {
        //                        var values = new List<string>();
        //                        values.Add(parentItem.Title);
        //                        selfAndParts.Add(selfParent.Title, values);
        //                    }
        //                }
        //                //没有部位
        //                else
        //                {
        //                    if (!selfAndParts.ContainsKey(selfParent.Title))
        //                    {
        //                        var values = new List<string>();
        //                        values.Add(string.Empty);
        //                        selfAndParts.Add(selfParent.Title, values);
        //                    }
        //                }
        //            }

        //            foreach (var selfAndPart in selfAndParts)
        //            {
        //                self.Add(selfAndPart.Key);
        //                selfAndPart.Value.RemoveAll(o => o.IsNullOrEmpty());
        //                part.AddRange(selfAndPart.Value);
        //            }
        //        }
        //        //没有色号的情况
        //        else
        //        {
        //            foreach (var primary in kv)
        //            {

        //                var detectionMethodDescription = "";
        //                if (primary.Metadata.ContainsKey("DetectionMethodDescription"))
        //                {
        //                    detectionMethodDescription = primary.Metadata["DetectionMethodDescription"].IsNullOrEmpty() ? "" : primary.Metadata["DetectionMethodDescription"];
        //                }
        //                var detectionCodeActual = primary.DetectionCodeActual;
        //                if (primary.Metadata.ContainsKey("DetectionCodeActual")) detectionCodeActual = primary.Metadata["DetectionCodeActual"];
        //                if (!detectionMethodDescription.IsNullOrEmpty())
        //                {
        //                    detectionCodeActual = detectionCodeActual.Replace(detectionMethodDescription, "");
        //                    detectionMethodDescription = " " + detectionMethodDescription;
        //                }
        //                if (!actual.Contains(primary.DetectionCodeActual)) actual.Add(detectionCodeActual);
        //                if (!method.Contains(detectionMethodDescription)) method.Add(detectionMethodDescription);

        //                var parentItem = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid);
        //                //说明有部位
        //                if (parentItem != null)
        //                {
        //                    if (!part.Contains(parentItem.Title))
        //                    {
        //                        part.Add(parentItem.Title);
        //                    }
        //                }
        //            }
        //        }

        //        self = self.Distinct().ToList();
        //        part = part.Distinct().ToList();
        //        actual = actual.Distinct().ToList();
        //        method = method.Distinct().ToList();

        //        if (!self.IsNullOrEmpty() && !part.IsNullOrEmpty())
        //        {

        //        }

        //        detectionStr += (self.IsNullOrEmpty() ? string.Empty : $"[{string.Join("、", self)}]") + (part.IsNullOrEmpty() ? string.Empty : $"({string.Join("、", part)})") + (actual.IsNullOrEmpty() ? (method.IsNullOrEmpty() ? string.Empty : "(" + string.Join("、", method) + ")") : "(" + string.Join("、", actual) + string.Join("、", method) + ")");
        //        testDetectionsList.Add(detectionStr);
        //    }
        //    testDetections = String.Join("、", testDetectionsList);
        //    report.TestDetections = testDetections;
        //    report.AddOnlyProperties(o => o.TestDetections);
        //    #endregion
        //    return report;
        //}

        public static void AutoSetReportItemAmount(this Report report,IDbConnection conn = null)
        {
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            //只能报告签发之后调用
            //if (report.Status <= 700) throw new Exception("报告尚未完成，无法自动同步费用！");
            var queryAllReportItem = conn.From<ReportItem>();
            queryAllReportItem.Where(o=>o.ReportKvid ==  report.Kvid);
            var allReportItems = conn.Select(queryAllReportItem);

            var mainReportItems = allReportItems.filterReportItem();
            foreach(var reportItem in mainReportItems)
            {
                if(reportItem.Type == ReportItemType.Test)
                {
                    if(reportItem.TestValue!=null&&(reportItem.TestValue == "/" || reportItem.TestValue.Contains("不考核") || reportItem.TestValue.Contains("不适用")))
                    {
                        reportItem.SetAmountLog(report, "自动核价", reportItem.Amount, 0, conn);
                        reportItem.SetAmountLog(report, "自动核价", reportItem.AmountPlan, 0, conn);
                        reportItem.AmountPlan = 0;
                        reportItem.Amount = 0;
                        reportItem.AddOnlyProperties(o => o.Amount);
                        reportItem.AddOnlyProperties(o => o.AmountPlan);
                        conn.UpdateOnly(reportItem);
                    }
                }
                else
                {
                    var subItems = allReportItems.Where(o=>o.ParentKvid == reportItem.Kvid);
                    if (subItems == null) continue;
                    //对比子项得数量和符合改价条件的子项数量
                    var countFilter = subItems.Count(o => o.TestValue == "/" || o.TestValue.Contains("不考核") || o.TestValue.Contains("不适用"));
                    if (countFilter == subItems.Count())
                    {
                        reportItem.SetAmountLog(report, "自动核价", reportItem.Amount, 0, conn);
                        reportItem.SetAmountLog(report, "自动核价", reportItem.AmountPlan, 0, conn);
                        reportItem.AmountPlan = 0;
                        reportItem.Amount = 0;
                        reportItem.AddOnlyProperties(o => o.Amount);
                        reportItem.AddOnlyProperties(o => o.AmountPlan);
                        conn.UpdateOnly(reportItem);
                    }
                }
            }
        }

        #region 日志相关
        public enum LogType
        {
            Unsupported = 0,
            //检验项目更新项目名称
            ReportItemUpdateTitle = 100,
            //检验项目更新标准值
            ReportItemUpdateDetectionValueQualified = 200,
            //检验项目更新数值单位
            ReportItemUpdateDetectionUnit = 300,
            //检验项目更新测试方法
            ReportItemUpdateDetectionCodeActual = 400
        }
        public static void CreateLog(this Report report, ReportItem reportItem, LogType logType, string logSummary = "", IDbConnection conn = null)
        {
            #region 数据验证
            report.ThrowIfNull("未传入报告信息!");
            reportItem.ThrowIfNull("未传入检验项目!");
            if (logType == LogType.Unsupported) throw new Exception("错误的日志类型!");
            #endregion
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();
            var log = new EntityLog<Report>();
            var bizId = string.Empty;
            var bizKvid = Guid.Empty;
            var bizType = string.Empty;
            var type = string.Empty;
            var title = string.Empty;
            var source = string.Empty;
            var summary = string.Empty;
            switch (logType)
            {
                case LogType.Unsupported:
                    break;
                case LogType.ReportItemUpdateTitle:
                    bizId = reportItem.Title;
                    bizKvid = reportItem.Kvid;
                    bizType = "Kivii.Lac.Lims.Entities.ReportItem";
                    type = "检验项目";
                    title = "项目名称更新";
                    source = reportItem.Title;
                    summary = logSummary;
                    break;
                case LogType.ReportItemUpdateDetectionValueQualified:
                    bizId = reportItem.Title;
                    bizKvid = reportItem.Kvid;
                    bizType = "Kivii.Lac.Lims.Entities.ReportItem";
                    type = "检验项目";
                    title = "标准值更新";
                    source = reportItem.Title;
                    summary = logSummary;
                    break;
                case LogType.ReportItemUpdateDetectionUnit:
                    bizId = reportItem.Title;
                    bizKvid = reportItem.Kvid;
                    bizType = "Kivii.Lac.Lims.Entities.ReportItem";
                    type = "检验项目";
                    title = "数值单位更新";
                    source = reportItem.Title;
                    summary = logSummary;
                    break;
                case LogType.ReportItemUpdateDetectionCodeActual:
                    bizId = reportItem.Title;
                    bizKvid = reportItem.Kvid;
                    bizType = "Kivii.Lac.Lims.Entities.ReportItem";
                    type = "检验项目";
                    title = "测试方法更新";
                    source = reportItem.Title;
                    summary = logSummary;
                    break;
                default:
                    break;
            }
            if (!source.IsNullOrEmpty() && source.Length > 50) source = source.Substring(0, 40) + "...";
            log.OwnerKvid = report.Kvid;
            log.BizId = bizId;
            log.BizKvid = bizKvid;
            log.BizType = bizType;
            log.Type = type;
            log.Title = title;
            log.Source = source;
            log.Summary = summary;
            conn.Insert(log);
        }
        #endregion

    }
}
