﻿using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lac.Lims.Transforms
{
    public class ReportStatisticResponse : RestfulQueryResponse<Report>
    {
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        /// <summary>
        /// 付款金额
        /// </summary>
        public decimal AmountPayment { get; set; }

        /// <summary>
        /// 结算金额
        /// </summary>
        public decimal AmountSettlement { get; set; }

        /// <summary>
        /// 报告费 (ServiceFee)
        /// </summary>
        public decimal AmountReport { get; set; }

        /// <summary>
        /// 更改报告费 (ChangeFee)
        /// </summary>
        public decimal AmountReportChange { get; set; }

        /// <summary>
        /// 检验费(Task表统计来,区别与TaskFee)
        /// </summary>
        public decimal AmountTask { get; set; }

        public decimal AmountUrgent { get; set; }

        /// <summary>
        /// 检验折后费(Task表统计来,区别与TasksDisCountFee)
        /// </summary>
        public decimal AmountTaskDiscount { get; set; }

        /// <summary>
        /// 套餐费(PackageFee)
        /// </summary>
        public decimal AmountPackage { get; set; }

        /// <summary>
        /// 应收检测费 AmountTask*UrgentRate
        /// </summary>
        public decimal AmountTaskTotal { get; set; }

        /// <summary>
        /// 实收检测费 AmountTaskDiscount*UrgentRate
        /// </summary>
        public decimal AmountTaskDiscountTotal { get; set; }

        /// <summary>
        /// 报告批次(不包含X报告)
        /// </summary>
        public int Quantity { get; set; }

        /// <summary>
        /// X报告批次
        /// </summary>
        public int XQuantity { get; set; }

        public object GetReportItems()
        {
            var reportKvids = new List<Guid>();
            var rtns = new List<ReportItem>();
            if (!Results.IsNullOrEmpty())
            {
                reportKvids.AddRange(Results.Select(o => o.Kvid));
                var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
                rtns = conn.Select<ReportItem>(o => Sql.In(o.ReportKvid, reportKvids));
            }
            foreach (var item in Results)
            {
                var details = rtns.Where(o => o.ReportKvid == item.Kvid).ToList();
                item.Tag = details.filterReportItem();
            }
            return rtns;
        }
    }

    [Api(Description = "报告合同统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportGroupQuotation : RestfulExecution<Report>
    {
        public Guid QuotationKvid { get; set; }
        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            QuotationKvid.ThrowIfEmpty("请传入要统计的合同信息!");
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var query = conn.From<Report>();
            query.Where(o => o.QuotationKvid == QuotationKvid && o.DealDate >= BeginTime && o.DealDate < EndTime);
            query.Select(o => new { o.QuotationName, o.DealDate, o.Amount, o.AmountPlan, o.AmountTest, o.AmountTestPlan, o.AmountSettlement, o.AmountPayment, o.QuotationAmount });
            query.OrderBy(o => o.DealDate);
            var reports = conn.Select(query);

            var rtns = new ReportStatisticResponse();
            rtns.Results = new List<Report>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            if (reports.IsNullOrEmpty()) return rtns;
            reports.ForEach(o =>//处理如果有协议费,那就把折后的检测费清除,方便最终统计
            {
                if (o.QuotationAmount > 0) o.AmountTest = 0;
            });
            var group = reports.GroupBy(o => new { o.QuotationName, Category = o.DealDate.Value.ToString("yyyy-MM") });
            foreach (var key in group)
            {
                var item = new Report();
                item.QuotationName = key.Key.QuotationName;
                item.Category = key.Key.Category;
                item.Quantity = key.Count();
                item.Amount = key.Sum(o => o.Amount);
                item.AmountPlan = key.Sum(o => o.AmountPlan);
                item.AmountTest = key.Sum(o => o.AmountTest);
                item.AmountTestPlan = key.Sum(o => o.AmountTestPlan);
                item.AmountSettlement = key.Sum(o => o.AmountSettlement);
                item.AmountPayment = key.Sum(o => o.AmountPayment);
                item.QuotationAmount = key.Sum(o => o.QuotationAmount);
                rtns.Results.Add(item);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
}
