﻿
using Kivii.DataAnnotations;
using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using Kivii.Lims;

namespace Kivii.Lac.Lims.Transforms
{
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "展示报告的检测项目（大项）")]
    public class ReportItemMain: RestfulExecution<ReportItem>
    {
        /// <summary>
        /// 要查看的项目报告Kvid,有部位信息则合并到大项中PartKvid和TitlePart
        /// </summary>
		[ApiMember(Description ="报告主键"),Required]
        public Guid ReportKvid { get; set; }

        /// <summary>
        /// 是否仅筛选传入的检验小组的检验项目,默认筛选全部项目
        /// </summary>
        public Guid WorkGroupKvid { get; set; }

        /// <summary>
        /// 是否是登录人所在组,默认false全部组
        /// </summary>
        public bool Owned { get; set; }

        /// <summary>
        /// 是否已经分组,默认不筛选
        /// </summary>
        public bool? Grouped { get; set; }

        /// <summary>
        /// 是否已经判定,默认不筛选
        /// </summary>
        public bool? Judged { get; set; }

        /// <summary>
        /// 仅显示主项,默认false,注:true时 上述分类无效
        /// 目前在报告结算时 明细清单使用到
        /// </summary>
        public bool OnlyMainItem { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("请指定要查看项目的报告Kvid!");

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            if (ReportKvid == Guid.Empty) return rtns;//为什么????

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == ReportKvid);
            if (allReportItems.IsNullOrEmpty()) return rtns;

            //全项中的大项
            var primaryAllReportItems = allReportItems.filterReportItem();

            List<ReportItem> finalResults = new List<ReportItem>();
            finalResults.AddRange(allReportItems);

            //只筛选主项出来
            if (OnlyMainItem)
            {
                finalResults = new List<ReportItem>();
                finalResults.AddRange(primaryAllReportItems);
            }
            else
            {
                //只查看登录人所在组
                if (Owned || WorkGroupKvid != Guid.Empty)
                {
                    finalResults = new List<ReportItem>();
                    //所在组的全部大项
                    List<ReportItem> ownerPrimaryItems = null;
                    if (WorkGroupKvid != Guid.Empty) ownerPrimaryItems = primaryAllReportItems.Where(o => o.WorkGroupKvid == WorkGroupKvid).ToList();
                    else if (Owned)
                    {
                        //获取duty表中对应的角色
                        var queryTestRoles = conn.From<EntityDuty<Test>>();
                        queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
                        queryTestRoles.Select(o => o.RoleName);
                        var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
                        ownerPrimaryItems = primaryAllReportItems.Where(o => o.WorkGroupKvid == KiviiContext.CurrentMember.DepartmentKvid || testRoleNames.Contains(o.WorkGroupName)).ToList();
                    }

                    //没找到直接返回了 
                    if (ownerPrimaryItems.IsNullOrEmpty()) return rtns;
                    //ownerPrimaryItems.ForEach(o =>
                    //{
                    //if (!finalResults.Exists(p => p.Kvid == o.Kvid)) finalResults.Add(o);
                    //});
                    finalResults.AddRange(ownerPrimaryItems);

                    var ownerNodeItems = allReportItems.Where(o => ownerPrimaryItems.ConvertAll(p => p.Kvid).Contains(o.ParentKvid)).ToList();
                    if (!ownerNodeItems.IsNullOrEmpty()) finalResults.AddRange(ownerNodeItems);

                    var ownerPartItems = allReportItems.Where(o => o.Type == ReportItemType.SamplePart && ownerPrimaryItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid)).ToList();

                    if (!ownerPartItems.IsNullOrEmpty())
                    {
                        finalResults.AddRange(ownerPartItems);

                        var ownerRootItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf && (ownerPartItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid) || ownerPrimaryItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid))).ToList();
                        if (!ownerRootItems.IsNullOrEmpty()) finalResults.AddRange(ownerRootItems);
                    }
                    else
                    {
                        var ownerRootItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf && ownerPrimaryItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid)).ToList();
                        if (!ownerRootItems.IsNullOrEmpty()) finalResults.AddRange(ownerRootItems);
                    }
                }
                //筛选是否已分组
                if (Grouped != null)
                {
                    var primaryItems = finalResults.filterReportItem();
                    if (!primaryItems.IsNullOrEmpty())
                    {
                        //只查已分组
                        if (Grouped.Value)
                        {
                            //只留下未分组的项目
                            primaryItems.RemoveAll(o => o.WorkGroupKvid != Guid.Empty);
                        }
                        //只查未分组
                        else
                        {
                            //只留下已分组的项目
                            primaryItems.RemoveAll(o => o.WorkGroupKvid == Guid.Empty);
                        }
                        //待移除的大项
                        var preRemoveKvids = primaryItems.ConvertAll(p => p.Kvid);
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.Kvid));
                        //也移除下面的子项
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.ParentKvid));
                    }
                }
                //筛选是否已判定
                if (Judged != null)
                {
                    var primaryItems = finalResults.filterReportItem();
                    if (!primaryItems.IsNullOrEmpty())
                    {
                        //只查已判定
                        if (Judged.Value)
                        {
                            //只留下未判定的项目
                            primaryItems.RemoveAll(o => !o.Judgement.IsNullOrEmpty());
                        }
                        //只查未判定
                        else
                        {
                            //只留下已判定的项目
                            primaryItems.RemoveAll(o => o.Judgement.IsNullOrEmpty());
                        }
                        //待移除的大项
                        var preRemoveKvids = primaryItems.ConvertAll(p => p.Kvid);
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.Kvid));
                        //也移除下面的子项
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.ParentKvid));
                    }
                }
            }

            #region 清理只有色号或部位的项
            //要是都没有检验项目 则清除当前Root下面的项目
            var group = finalResults.GroupBy(o => o.RootKvid);
            foreach (var kv in group)
            {
                if (!kv.ToList().Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) finalResults.RemoveAll(o => o.RootKvid == kv.Key);
            }
            #endregion

            #region 合并部位信息
            rtns.Results.AddRange(finalResults);
            var sampleParts = rtns.Results.Where(o => o.Type == ReportItemType.SamplePart).ToList();
            if (sampleParts.IsNullOrEmpty())
            {
                //要是最后都没有检验项目返回了 则清除全部项目
                if (!rtns.Results.Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) rtns.Results.Clear();
                //var newList2 = new List<ReportItem>();
                //foreach (var item in rtns.Results)
                //{
                //    if (newList2.Exists(o => o.Kvid == item.Kvid)) continue;
                //    newList2.Add(item);
                //}
                //rtns.Results = newList2;
                rtns.Total = rtns.Results.Count;

                return rtns;
            }

            foreach (var samplePart in sampleParts)
            {
                var tests = rtns.Results.Where(o => o.ParentKvid == samplePart.Kvid).ToList();
                foreach (var test in tests)
                {
                    test.TitlePart = samplePart.Title;
                    test.ParentKvid = samplePart.ParentKvid;
                    test.PartKvid = samplePart.Kvid;
                }
            }

            rtns.Results.RemoveAll(o => sampleParts.ConvertAll(p => p.Kvid).Contains(o.Kvid));
            #endregion

            //要是最后都没有检验项目返回了 则清除全部项目
            if (!rtns.Results.Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) rtns.Results.Clear();

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }


    #region 不合格项目统计[使用模板方式]
    public class ReportItemUnqualifiedQuery : RestfulExecution<ReportItem>
    {
        #region 报告查询参数
        public DateTime? DealDateFrom { get; set; }
        public DateTime? DealDateTo { get; set; }
        public string DemanderName { get; set; }
        public string SampleManufacturerName { get; set; }
        public string PayerName { get; set; }
        public string ReportIdNumber { get; set; }
        public string SampleName { get; set; }
        public string ReportItemTitle { get; set; }

        public bool JudgementNoPass { get; set; }//筛选判定不合格的项目
        #endregion

        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (DealDateFrom == null || DealDateTo == null) throw new Exception("参数委托日期范围不能为空！");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<ReportItemEx>();
            request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Rows = null;
            sqlExpress
                .LeftJoin<Report>((reportItemEx, report) => reportItemEx.ReportKvid == report.Kvid)
                .Select<ReportItemEx, Report>((reportItemEx, report) => new
                {
                    T = Sql.AllFields(reportItemEx),
                    report.ReportId,
                    ReportDemanderName = report.DemanderName,
                    ReportDealDate = report.DealDate,
                    ReportDeadDate = report.DeadDate,
                    ReportSampleName = report.SampleName,
                    ReportSampleBatchNumber = report.SampleBatchNumber,
                    ReportSampleBrand = report.SampleBrand,
                    ReportPayerName = report.PayerName,
                    ReportMetadataString = report.Metadata
                    //ReportAttachDocument = report.Metadata.ContainsKey("DescriptionSpecial") ? report.Metadata["DescriptionSpecial"] : "",
                });
            if (DealDateFrom != null)
            {
                sqlExpress.And<Report>(o => o.DealDate >= DealDateFrom);
            }
            if (DealDateTo != null)
            {
                sqlExpress.And<Report>(o => o.DealDate <= DealDateTo);
            }
            if (!DemanderName.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.DemanderName.Contains(DemanderName));
            }
            if (!SampleManufacturerName.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.SampleManufacturerName.Contains(SampleManufacturerName));
            }
            if (!PayerName.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.PayerName.Contains(PayerName));
            }
            if (!ReportIdNumber.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.ReportId.Contains(ReportIdNumber));
            }
            if (!SampleName.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.SampleName.Contains(SampleName));
            }
            if (!ReportItemTitle.IsNullOrEmpty())
            {
                sqlExpress.And<ReportItemEx>(o => o.Title.Contains(ReportItemTitle));
            }
            //sqlExpress.And<ReportItemEx>(o => o.Judgement == "不合格" || o.Judgement == "不符合");
            if (JudgementNoPass) sqlExpress.And<ReportItemEx>(o => Sql.In(o.Judgement, Configs.JudgementNoPassResults));
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            //根据查询结果中的大项，找出子项
            var nodeParentKvid = rtns.Results.Where(o => o.Type == ReportItemType.TestFolder).ToList().ConvertAll(o => o.Kvid).Distinct().ToList();
            var queryNode = conn.From<ReportItem>();
            queryNode.Where(o => o.Type == ReportItemType.Test && Sql.In(o.ParentKvid, nodeParentKvid));
            if (JudgementNoPass) queryNode.And<ReportItem>(o => Sql.In(o.Judgement, Configs.JudgementNoPassResults));
            var nodeList = conn.Select(queryNode);

            //根据查询结果中的parentKvid找出检验项目色号或部位数据//增加大项的，避免检索出结果无大项
            var uniqueParentKvid = rtns.Results.ConvertAll(o => o.ParentKvid).Distinct().ToList();
            var queryParent = conn.From<ReportItem>();
            queryParent.Where(o => (o.Type == ReportItemType.SampleSelf || o.Type == ReportItemType.SamplePart || o.Type == ReportItemType.TestFolder) && Sql.In(o.Kvid, uniqueParentKvid));
            var parentList = conn.Select(queryParent);

            //可能存在大项的父项，需要再查一次
            var uniqueGrandParentKvid = parentList.ConvertAll(o => o.ParentKvid).Distinct().ToList();
            var queryGrandParent = conn.From<ReportItem>();
            queryGrandParent.Where(o => (o.Type == ReportItemType.SampleSelf || o.Type == ReportItemType.SamplePart) & Sql.In(o.Kvid, uniqueGrandParentKvid));
            var grandParentList = conn.Select(queryGrandParent);

            //可能存在部位的父项，需要再查一次
            var uniqueGrandGrandParentKvid = grandParentList.ConvertAll(o => o.ParentKvid).Distinct().ToList();
            var queryGrandGrandParent = conn.From<ReportItem>();
            queryGrandGrandParent.Where(o => (o.Type == ReportItemType.SampleSelf) & Sql.In(o.Kvid, uniqueGrandGrandParentKvid));
            var grandGrandParentList = conn.Select(queryGrandGrandParent);

            //合并
            var allParent = new List<ReportItem>();
            allParent.AddRange(parentList);
            allParent.AddRange(grandParentList);
            allParent.AddRange(grandGrandParentList);
            allParent = allParent.Distinct().ToList();
            //sqlExpress.OrderBy(o => o.ReportKvid);

            var rtnResults = new List<ReportItemEx>();
            rtnResults.AddRange(rtns.Results);
            //遍历返回的检验项目，从合并的父项中匹配色号或部位信息
            foreach (var item in rtns.Results)
            {
                var parent = allParent.FirstOrDefault(o => o.Kvid == item.ParentKvid);
                if (parent != null)
                {
                    if (parent.Type == ReportItemType.TestFolder)
                    {
                        if (rtnResults.Exists(o => o.Kvid == parent.Kvid))
                        {
                            item.DetectionCodeActual = parent.DetectionCodeActual;
                            item.StandardCode = parent.StandardCode;
                            continue;
                        }
                        var addItem = new ReportItemEx();
                        addItem.PopulateWith(parent);
                        var grandParent = allParent.FirstOrDefault(o => o.Kvid == parent.ParentKvid);
                        if (grandParent != null)
                        {
                            if (grandParent.Type == ReportItemType.SamplePart)
                            {
                                var grandgrandParent = allParent.FirstOrDefault(o => o.Kvid == grandParent.ParentKvid);//色号
                                if (grandgrandParent != null)
                                {
                                    addItem.TitlePart = grandParent.Title;
                                    addItem.TitleSelf = grandgrandParent.Title;
                                    //addItem.Title = $"{addItem.Title}[{grandgrandParent.Title}]({grandParent.Title})";
                                }
                                else
                                {
                                    addItem.TitlePart = grandParent.Title;
                                    //addItem.Title = $"{addItem.Title}({grandParent.Title})";
                                }
                            }
                            else
                            {
                                addItem.TitleSelf = grandParent.Title;
                                //addItem.Title = $"{addItem.Title}[{grandParent.Title}]";
                            }
                        }
                        addItem.ParentKvid = Guid.Empty;
                        addItem.RootKvid = Guid.Empty;
                        addItem.ReportId = item.ReportId;
                        addItem.ReportDemanderName = item.ReportDemanderName;
                        addItem.ReportDealDate = item.ReportDealDate;
                        addItem.ReportDeadDate = item.ReportDeadDate;
                        addItem.ReportSampleName = item.ReportSampleName;
                        addItem.ReportSampleBatchNumber = item.ReportSampleBatchNumber;
                        addItem.ReportSampleBrand = item.ReportSampleBrand;
                        addItem.ReportPayerName = item.ReportPayerName;
                        addItem.ReportMetadataString = item.ReportMetadataString;
                        item.DetectionCodeActual = addItem.DetectionCodeActual;
                        item.StandardCode = addItem.StandardCode;
                        item.TitlePart = addItem.TitlePart;
                        item.TitleSelf = addItem.TitleSelf;
                        rtnResults.Add(addItem);
                    }
                    else if (parent.Type == ReportItemType.SamplePart)
                    {
                        var grandParent = allParent.FirstOrDefault(o => o.Kvid == parent.ParentKvid);//色号
                        if (grandParent != null)
                        {
                            item.TitlePart = parent.Title;
                            item.TitleSelf = grandParent.Title;
                            //item.Title = $"{item.Title}[{grandParent.Title}]({parent.Title})";
                        }
                        else
                        {
                            item.TitlePart = parent.Title;
                            //item.Title = $"{item.Title}({parent.Title})";
                        }
                        item.ParentKvid = Guid.Empty;
                        item.RootKvid = Guid.Empty;
                    }
                    else
                    {
                        item.TitleSelf = parent.Title;
                        //item.Title = $"{item.Title}[{parent.Title}]";
                        item.ParentKvid = Guid.Empty;
                        item.RootKvid = Guid.Empty;
                    }
                }
                var nodes = nodeList.Where(o => o.ParentKvid == item.Kvid).ToList();
                if (!nodes.IsNullOrEmpty())
                {
                    foreach (var node in nodes)
                    {
                        if (rtnResults.Exists(o => o.Kvid == node.Kvid)) continue;
                        var addItem = new ReportItemEx();
                        addItem.PopulateWith(node);
                        addItem.ReportId = item.ReportId;
                        addItem.ReportDemanderName = item.ReportDemanderName;
                        addItem.ReportDealDate = item.ReportDealDate;
                        addItem.ReportDeadDate = item.ReportDeadDate;
                        addItem.ReportSampleName = item.ReportSampleName;
                        addItem.ReportSampleBatchNumber = item.ReportSampleBatchNumber;
                        addItem.ReportSampleBrand = item.ReportSampleBrand;
                        addItem.ReportPayerName = item.ReportPayerName;
                        addItem.ReportMetadataString = item.ReportMetadataString;
                        addItem.TitlePart = item.TitlePart;
                        addItem.TitleSelf = item.TitleSelf;
                        addItem.DetectionCodeActual = item.DetectionCodeActual;
                        addItem.StandardCode = item.StandardCode;
                        rtnResults.Add(addItem);
                    }
                }
            }
            rtnResults = rtnResults.Distinct().OrderBy(o => o.ReportId).ToList();
            var treeItems = rtnResults.getTreeItemsEx(Guid.Empty);
            var resPlatResults = new List<ReportItemEx>();
            treeItems.getPlatItemsEx(resPlatResults);
            resPlatResults = resPlatResults.OrderBy(o => o.ReportId).ToList();
            return resPlatResults;
        }
    }
    #endregion
}
