﻿using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lac.Lims.Transforms
{
    #region CDUQD
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class SampleCreate : RestfulCreate<Sample> { }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class SampleDelete : RestfulDelete<Sample> { }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class SampleUpdate : RestfulUpdate<Sample> { }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class SampleQuery : RestfulQuery<Sample> { }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class SampleRead : RestfulRead<Sample> { }
    #endregion

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleBindImages : RestfulExecution<Sample>
    {
        public Guid DbFileKvid { get; set; }
        public List<Guid> DbFileKvids { get; set; }

        public Guid ReportKvid { get; set; }

        public bool AutoDelete { get; set; } = true;

        public override object OnExecution(IRequest req, IResponse res)
        {
            (DbFileKvid == Guid.Empty && DbFileKvids.IsNullOrEmpty()).ThrowIfTrue("Need DbFileKvid Or DbFileKvids!");
            if (DbFileKvids.IsNullOrEmpty()) DbFileKvids = new List<Guid>();
            if (DbFileKvid != Guid.Empty) DbFileKvids.Add(DbFileKvid);
            if (ReportKvid == Guid.Empty) throw new Exception("缺少关联报告信息");

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            if (report == null) throw new Exception("未找到此报告信息！");
            conn.InitEntityType<EntityDbFile<Sample>>();

            var dbFiles = conn.SelectByIds<EntityDbFile<Sample>>(DbFileKvids);
            if (dbFiles.IsNullOrEmpty()) throw new Exception("未找到要关联的图片");

            var rtns = new RestfulExecutionResponse<EntityDbFile<Sample>>();
            rtns.Results = new List<EntityDbFile<Sample>>();
            foreach (var item in dbFiles)
            {
                var filePath = item.GetPhysicalPath();
                Stream mStream = File.OpenRead(filePath);

                var sPath = $"/Storages{Configs.RouteReportSampleFilePath}/{report.SampleKvid}/{item.Name}";
                var virtualFile = KiviiContext.VirtualFiles.WriteFile(sPath, mStream);

                mStream.Close();
                mStream.Dispose();

                if (AutoDelete) conn.DeleteById<EntityDbFile<Sample>>(item.Kvid);
                rtns.Results.Add(item);
            }
            return rtns;
        }
    }
    #region 新建报告绑定Sample功能
    //[RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    //[Api(Description = "样品绑定")]
    //public class SampleBind : RestfulExecution<Sample>
    //{
    //    public Guid SampleKvid { get; set; }
    //    public Report Report { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        //验证数据有效性
    //        var conn = KiviiContext.GetOpenedDbConnection<Report>();
    //        var sample = conn.SingleById<Sample>(SampleKvid);
    //        sample.ThrowIfNull("未查询到绑定的样品信息！");
    //        if (Report.Kvid != Guid.Empty) throw new Exception("当前报告已关联了样品，无法绑定新的样品！");
    //        var rtns = new RestfulCreateResponse<Report>();
    //        var trans = conn.OpenTransaction();
    //        try
    //        {
    //            //创建report
    //            var report = new Report();
    //            report.PopulateWith(Report);
    //            report.Kvid = Guid.NewGuid();
    //            report.ReportId = string.Empty;
    //            report.SampleBatchNumber = sample.BatchNumber;
    //            report.SampleBrand = sample.Brand;
    //            report.SampleGrade = sample.Grade;
    //            report.SampleGradeSafety = sample.GradeSafety;
    //            report.SampleKvid = sample.Kvid;
    //            report.SampleManufacturerAddress = sample.ManufacturerAddress;
    //            report.SampleManufacturerContactKvid = sample.ManufacturerContactKvid;
    //            report.SampleManufacturerContactName = sample.ManufacturerContactName;
    //            report.SampleManufacturerContactNumber = sample.ManufacturerContactNumber;
    //            report.SampleManufacturerKvid = sample.ManufacturerKvid;
    //            report.SampleManufacturerName = sample.ManufacturerName;
    //            report.SampleModel = sample.Model;
    //            report.SampleName = sample.Name;
    //            report.SampleQuantity = sample.Quantity;
    //            report.SampleQuantityUnit = sample.QuantityUnit;
    //            conn.Insert(report);
    //            report.RemoveAllOnlyProperties();

    //            rtns.Results = new List<Report>();
    //            rtns.Results.Add(report);

    //            //查询Test
    //            var queryTests = conn.From<Test>();
    //            queryTests.Where(o => o.SampleKvid == SampleKvid);
    //            var tests = conn.Select(queryTests);
    //            //创建ReportJudgement
    //            foreach (var test in tests)
    //            {
    //                var reportJudgement = new ReportJudgement();
    //                reportJudgement.ReportKvid = report.Kvid;
    //                reportJudgement.TestKvid = test.Kvid;
    //                reportJudgement.ValueUnit = test.ValueUnit;
    //                reportJudgement.ValueQualified = test.ValueQualified;
    //                conn.Insert(reportJudgement);
    //            }
    //            trans?.Commit();
    //        }
    //        catch(Exception ex)
    //        {
    //            trans?.Rollback();
    //            throw ex;
    //        }
    //        return rtns;
    //    }
    //}
    #endregion
}
